@@ -20,7 +20,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +38,7 @@
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
-/*#include <libedata-book/libedata-book.h>*/
+#include <libedata-book/libedata-book.h>
 
 #include "lzx/ews-oal-decompress.h"
 
@@ -48,7 +51,6 @@
 #include "utils/e-ews-query-to-restriction.h"
 
 #include "e-book-backend-ews.h"
-#include "ews-book-backend-sqlitedb.h"
 #include "ews-oab-decoder.h"
 
 #define d(x) x
@@ -59,8 +61,8 @@
 static gboolean ebews_fetch_items (EBookBackendEws *ebews,  GSList *items, gboolean store_to_cache, GSList **vcards, GCancellable *cancellable, GError **error);
 
 typedef struct {
-	GCond *cond;
-	GMutex *mutex;
+	GCond cond;
+	GMutex mutex;
 	gboolean exit;
 } SyncDelta;
 
@@ -70,7 +72,7 @@ struct _EBookBackendEwsPrivate {
 	gchar *oab_url;
 	gchar *folder_name;
 
-	EwsBookBackendSqliteDB *ebsdb;
+	EBookBackendSqliteDB *summary;
 
 	gboolean only_if_exists;
 	gboolean is_writable;
@@ -83,7 +85,7 @@ struct _EBookBackendEwsPrivate {
 	/* used for storing attachments */
 	gchar *attachment_dir;
 
-	GStaticRecMutex rec_mutex;
+	GRecMutex rec_mutex;
 	GThread *dthread;
 	SyncDelta *dlock;
 
@@ -107,8 +109,8 @@ enum {
  * some additional properties that are not return with Default view */
 #define CONTACT_ITEM_PROPS "item:Attachments item:HasAttachments contacts:Manager contacts:Department contacts:SpouseName contacts:AssistantName contacts:BusinessHomePage contacts:Birthday"
 
-#define PRIV_LOCK(p)   (g_static_rec_mutex_lock (&(p)->rec_mutex))
-#define PRIV_UNLOCK(p) (g_static_rec_mutex_unlock (&(p)->rec_mutex))
+#define PRIV_LOCK(p)   (g_rec_mutex_lock (&(p)->rec_mutex))
+#define PRIV_UNLOCK(p) (g_rec_mutex_unlock (&(p)->rec_mutex))
 
 /* Forward Declarations */
 static void	e_book_backend_ews_authenticator_init
@@ -924,7 +926,7 @@ ews_create_contact_cb (GObject *object,
 	/* get a list of ids from server (single item) */
 	e_ews_connection_create_items_finish (cnc, res, &items, &error);
 
-	g_return_if_fail (ebews->priv->ebsdb != NULL);
+	g_return_if_fail (ebews->priv->summary != NULL);
 
 	if (error == NULL) {
 		EEwsItem *item = (EEwsItem *) items->data;
@@ -934,7 +936,7 @@ ews_create_contact_cb (GObject *object,
 
 		e_contact_set (create_contact->contact, E_CONTACT_UID, item_id->id);
 		e_contact_set (create_contact->contact, E_CONTACT_REV, item_id->change_key);
-		ews_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, create_contact->contact, FALSE, &error);
+		e_book_backend_sqlitedb_add_contact (ebews->priv->summary, ebews->priv->folder_id, create_contact->contact, FALSE, &error);
 
 		if (error == NULL) {
 			GSList *contacts;
@@ -1057,10 +1059,10 @@ ews_book_remove_contact_cb (GObject *object,
 
 	simple = G_SIMPLE_ASYNC_RESULT (res);
 
-	g_return_if_fail (priv->ebsdb != NULL);
+	g_return_if_fail (priv->summary != NULL);
 
 	if (!g_simple_async_result_propagate_error (simple, &error))
-		deleted = ews_book_backend_sqlitedb_remove_contacts (priv->ebsdb, priv->folder_id, remove_contact->sl_ids, &error);
+		deleted = e_book_backend_sqlitedb_remove_contacts (priv->summary, priv->folder_id, remove_contact->sl_ids, &error);
 
 	if (deleted)
 		e_data_book_respond_remove_contacts (remove_contact->book, remove_contact->opid, EDB_ERROR (SUCCESS),  remove_contact->sl_ids);
@@ -1159,7 +1161,7 @@ ews_modify_contact_cb (GObject *object,
 
 	e_ews_connection_update_items_finish (cnc, res, &items, &error);
 
-	g_return_if_fail (priv->ebsdb != NULL);
+	g_return_if_fail (priv->summary != NULL);
 
 	if (error == NULL) {
 		EEwsItem *item = (EEwsItem *) items->data;
@@ -1172,8 +1174,8 @@ ews_modify_contact_cb (GObject *object,
 
 		id = e_contact_get (modify_contact->old_contact, E_CONTACT_UID);
 
-		ews_book_backend_sqlitedb_remove_contact (priv->ebsdb, priv->folder_id, id, &error);
-		ews_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, modify_contact->new_contact, FALSE, &error);
+		e_book_backend_sqlitedb_remove_contact (priv->summary, priv->folder_id, id, &error);
+		e_book_backend_sqlitedb_add_contact (ebews->priv->summary, ebews->priv->folder_id, modify_contact->new_contact, FALSE, &error);
 
 		if (error == NULL) {
 			GSList *new_contacts;
@@ -1289,7 +1291,7 @@ e_book_backend_ews_modify_contacts (EBookBackend *backend,
 		return;
 	}
 
-	g_return_if_fail (priv->ebsdb != NULL);
+	g_return_if_fail (priv->summary != NULL);
 
 	contact = e_contact_new_from_vcard (vcards->data);
 
@@ -1305,8 +1307,8 @@ e_book_backend_ews_modify_contacts (EBookBackend *backend,
 		return;
 	}
 
-	old_contact = ews_book_backend_sqlitedb_get_contact (
-		priv->ebsdb, priv->folder_id,
+	old_contact = e_book_backend_sqlitedb_get_contact (
+		priv->summary, priv->folder_id,
 		id->id, NULL, NULL, &error);
 	if (!old_contact) {
 		g_object_unref (contact);
@@ -1371,14 +1373,14 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 	priv = ebews->priv;
 
 	if (!e_backend_get_online (E_BACKEND (backend))) {
-		if (priv->ebsdb && ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
-			list = ews_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
+		if (priv->summary && e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL)) {
+			list = e_book_backend_sqlitedb_search (priv->summary, priv->folder_id, query, NULL, NULL, NULL, &error);
 			l = list;
 			while (l) {
-				EwsSdbSearchData *s_data = (EwsSdbSearchData *) l->data;
+				EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
 
 				vcard_list = g_slist_append (vcard_list, g_strdup (s_data->vcard));
-				ews_book_backend_sqlitedb_search_data_free (s_data);
+				e_book_backend_sqlitedb_search_data_free (s_data);
 				l = l->next;
 			}
 			convert_error_to_edb_error (&error);
@@ -1398,14 +1400,14 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 		return;
 	}
 
-	if (priv->ebsdb && ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
-		list = ews_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
+	if (priv->summary && e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL)) {
+		list = e_book_backend_sqlitedb_search (priv->summary, priv->folder_id, query, NULL, NULL, NULL, &error);
 		l = list;
 		while (l) {
-			EwsSdbSearchData *s_data = (EwsSdbSearchData *) l->data;
+			EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
 
 			vcard_list = g_slist_append (vcard_list, g_strdup (s_data->vcard));
-			ews_book_backend_sqlitedb_search_data_free (s_data);
+			e_book_backend_sqlitedb_search_data_free (s_data);
 			l = l->next;
 		}
 
@@ -1663,10 +1665,10 @@ ews_gal_needs_update (EBookBackendEws *cbews,
 	gboolean ret = FALSE;
 	gchar *tmp;
 
-	if (!priv->ebsdb)
+	if (!priv->summary)
 		return FALSE;
 
-	tmp = ews_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "seq", error);
+	tmp = e_book_backend_sqlitedb_get_key_value (priv->summary, priv->folder_id, "seq", error);
 	if (error)
 		goto exit;
 
@@ -1741,10 +1743,10 @@ ews_remove_old_gal_file (EBookBackendEws *cbews,
 	EBookBackendEwsPrivate *priv = cbews->priv;
 	gchar *filename;
 
-	if (!priv->ebsdb)
+	if (!priv->summary)
 		return FALSE;
 
-	filename = ews_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "oab-filename", error);
+	filename = e_book_backend_sqlitedb_get_key_value (priv->summary, priv->folder_id, "oab-filename", error);
 	if (*error)
 		return FALSE;
 
@@ -1771,7 +1773,7 @@ ews_gal_store_contact (EContact *contact,
 	struct _db_data *data = (struct _db_data *) user_data;
 	EBookBackendEwsPrivate *priv = data->cbews->priv;
 
-	g_return_if_fail (priv->ebsdb != NULL);
+	g_return_if_fail (priv->summary != NULL);
 
 	data->contact_collector = g_slist_prepend (data->contact_collector, g_object_ref (contact));
 	data->collected_length += 1;
@@ -1787,7 +1789,7 @@ ews_gal_store_contact (EContact *contact,
 		g_free (status_message);
 
 		data->contact_collector = g_slist_reverse (data->contact_collector);
-		ews_book_backend_sqlitedb_add_contacts (priv->ebsdb, priv->folder_id, data->contact_collector, FALSE, error);
+		e_book_backend_sqlitedb_add_contacts (priv->summary, priv->folder_id, data->contact_collector, FALSE, error);
 
 		for (l = data->contact_collector; l != NULL; l = g_slist_next (l))
 			e_book_backend_notify_update (E_BOOK_BACKEND (data->cbews), E_CONTACT (l->data));
@@ -1813,14 +1815,19 @@ ews_replace_gal_in_db (EBookBackendEws *cbews,
 	gboolean ret = TRUE;
 	struct _db_data data;
 
-	g_return_val_if_fail (priv->ebsdb != NULL, FALSE);
+	g_return_val_if_fail (priv->summary != NULL, FALSE);
 
 	/* remove the old address-book and create a new one in db */
-	if (ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
-		ret = ews_book_backend_sqlitedb_delete_addressbook (priv->ebsdb, priv->folder_id, error);
+	if (e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL)) {
+		GSList *uids;
+
+		uids = e_book_backend_sqlitedb_search_uids (priv->summary, priv->folder_id, NULL, NULL, NULL);
+		if (uids) {
+			e_book_backend_sqlitedb_remove_contacts (priv->summary, priv->folder_id, uids, NULL);
+			g_slist_free_full (uids, g_free);
+		}
+
 		ews_remove_attachments (priv->attachment_dir);
-		if (ret)
-			ret = ews_book_backend_sqlitedb_create_addressbook (priv->ebsdb, priv->folder_id, priv->folder_name, TRUE, error);
 	}
 
 	if (!ret)
@@ -1839,7 +1846,7 @@ ews_replace_gal_in_db (EBookBackendEws *cbews,
 	       return ret;
 
 	/* mark the db as populated */
-	ret = ews_book_backend_sqlitedb_set_is_populated (priv->ebsdb, priv->folder_id, TRUE, error);
+	ret = e_book_backend_sqlitedb_set_is_populated (priv->summary, priv->folder_id, TRUE, error);
 
 	return ret;
 }
@@ -1877,8 +1884,8 @@ ebews_start_gal_sync (gpointer data)
 		goto exit;
 	}
 
-	g_warn_if_fail (priv->ebsdb != NULL);
-	if (!priv->ebsdb)
+	g_warn_if_fail (priv->summary != NULL);
+	if (!priv->summary)
 		goto exit;
 
 	if (full_l == NULL)
@@ -1886,7 +1893,7 @@ ebews_start_gal_sync (gpointer data)
 
 	full = (EwsOALDetails *) full_l->data;
 	/* TODO fetch differential updates if available instead of downloading the whole GAL */
-	if (!ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL) || ews_gal_needs_update (cbews, full, &error)) {
+	if (!e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL) || ews_gal_needs_update (cbews, full, &error)) {
 		gchar *seq;
 
 		d (printf ("Ewsgal: Downloading full gal \n");)
@@ -1909,11 +1916,11 @@ ebews_start_gal_sync (gpointer data)
 			goto exit;
 
 		seq = g_strdup_printf ("%"G_GUINT32_FORMAT, full->seq);
-		ret = ews_book_backend_sqlitedb_set_key_value (priv->ebsdb, priv->folder_id, "seq", seq, &error);
+		ret = e_book_backend_sqlitedb_set_key_value (priv->summary, priv->folder_id, "seq", seq, &error);
 		g_free (seq);
 
 		if (!ret) {
-			ews_book_backend_sqlitedb_delete_addressbook (priv->ebsdb, priv->folder_id, &error);
+			e_book_backend_sqlitedb_delete_addressbook (priv->summary, priv->folder_id, &error);
 			goto exit;
 		}
 	}
@@ -1961,14 +1968,14 @@ ebews_sync_deleted_items (EBookBackendEws *ebews,
 
 	priv = ebews->priv;
 
-	g_return_if_fail (priv->ebsdb != NULL);
+	g_return_if_fail (priv->summary != NULL);
 
 	for (l = deleted_ids; l != NULL; l = g_slist_next (l)) {
 		gchar *id = (gchar *) l->data;
 		gboolean partial_content;
 
-		if (ews_book_backend_sqlitedb_has_contact (priv->ebsdb, priv->folder_id, id, &partial_content, NULL))
-			ews_book_backend_sqlitedb_remove_contact (priv->ebsdb, priv->folder_id, id, error);
+		if (e_book_backend_sqlitedb_has_contact (priv->summary, priv->folder_id, id, &partial_content, NULL))
+			e_book_backend_sqlitedb_remove_contact (priv->summary, priv->folder_id, id, error);
 		e_book_backend_notify_remove (E_BOOK_BACKEND (ebews), id);
 	}
 
@@ -1987,7 +1994,7 @@ ebews_store_contact_items (EBookBackendEws *ebews,
 
 	priv = ebews->priv;
 
-	g_return_if_fail (priv->ebsdb != NULL);
+	g_return_if_fail (priv->summary != NULL);
 
 	for (l = new_items; l != NULL; l = g_slist_next (l)) {
 		EContact *contact;
@@ -2018,7 +2025,7 @@ ebews_store_contact_items (EBookBackendEws *ebews,
 			/* store display_name, fileas, item id */	
 		}
 
-		ews_book_backend_sqlitedb_add_contact (priv->ebsdb, priv->folder_id, contact, FALSE, error);
+		e_book_backend_sqlitedb_add_contact (priv->summary, priv->folder_id, contact, FALSE, error);
 		e_book_backend_notify_update (E_BOOK_BACKEND (ebews), contact);
 
 		g_object_unref (item);
@@ -2076,7 +2083,7 @@ ebews_store_distribution_list_items (EBookBackendEws *ebews,
 	GSList *l;
 	EContact *contact;
 
-	g_return_if_fail (ebews->priv->ebsdb != NULL);
+	g_return_if_fail (ebews->priv->summary != NULL);
 
 	contact = e_contact_new ();
 	e_contact_set (contact, E_CONTACT_UID, id->id);
@@ -2110,7 +2117,7 @@ ebews_store_distribution_list_items (EBookBackendEws *ebews,
 	}
 
 	g_slist_free (members);
-	ews_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, contact, FALSE, error);
+	e_book_backend_sqlitedb_add_contact (ebews->priv->summary, ebews->priv->folder_id, contact, FALSE, error);
 	e_book_backend_notify_update (E_BOOK_BACKEND (ebews), contact);
 
 	g_object_unref (contact);
@@ -2280,6 +2287,30 @@ cleanup:
 	return TRUE;
 }
 
+static void
+ebews_forget_all_contacts (EBookBackendEws *ebews)
+{
+	EBookBackend *backend;
+	GSList *ids = NULL;
+
+	g_return_if_fail (E_IS_BOOK_BACKEND_EWS (ebews));
+
+	backend = E_BOOK_BACKEND (ebews);
+	g_return_if_fail (backend != NULL);
+
+	ids = e_book_backend_sqlitedb_search_uids (ebews->priv->summary, ebews->priv->folder_id, NULL, NULL, NULL);
+	if (ids) {
+		GSList *id;
+
+		e_book_backend_sqlitedb_remove_contacts (ebews->priv->summary, ebews->priv->folder_id, ids, NULL);
+		for (id = ids; id; id = id->next) {
+			e_book_backend_notify_remove (backend, id->data);
+		}
+
+		g_slist_free_full (ids, g_free);
+	}
+}
+
 static gboolean
 ebews_start_sync (gpointer data)
 {
@@ -2292,7 +2323,7 @@ ebews_start_sync (gpointer data)
 	ebews = (EBookBackendEws *) data;
 	priv = ebews->priv;
 
-	g_return_val_if_fail (priv->ebsdb != NULL, FALSE);
+	g_return_val_if_fail (priv->summary != NULL, FALSE);
 
 	/* Not connected? Try again later */
 	if (!priv->cnc)
@@ -2302,20 +2333,36 @@ ebews_start_sync (gpointer data)
 	e_book_backend_foreach_view (E_BOOK_BACKEND (ebews), book_view_notify_status, status_message);
 	g_free (status_message);
 
-	sync_state = ews_book_backend_sqlitedb_get_sync_data (priv->ebsdb, priv->folder_id, NULL);
+	sync_state = e_book_backend_sqlitedb_get_sync_data (priv->summary, priv->folder_id, NULL);
 	do
 	{
 		GSList *items_created = NULL, *items_updated = NULL;
 		GSList *items_deleted = NULL;
+		gchar *old_sync_state = sync_state;
+
+		sync_state = NULL;
 
 		e_ews_connection_sync_folder_items_sync (
 			priv->cnc, EWS_PRIORITY_MEDIUM,
-			&sync_state, priv->folder_id,
-			"IdOnly", NULL,
-			EWS_MAX_FETCH_COUNT, &includes_last_item,
+			old_sync_state, priv->folder_id,
+			"IdOnly", NULL, EWS_MAX_FETCH_COUNT,
+			&sync_state,
+			&includes_last_item,
 			&items_created, &items_updated,
 			&items_deleted, priv->cancellable, &error);
 
+		g_free (old_sync_state);
+
+		if (g_error_matches (error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_INVALIDSYNCSTATEDATA)) {
+			g_clear_error (&error);
+			e_book_backend_sqlitedb_set_sync_data (priv->summary, priv->folder_id, NULL, &error);
+			ebews_forget_all_contacts (ebews);
+
+			e_ews_connection_sync_folder_items_sync (priv->cnc, EWS_PRIORITY_MEDIUM, NULL, priv->folder_id, "IdOnly", NULL, EWS_MAX_FETCH_COUNT,
+				&sync_state, &includes_last_item, &items_created, &items_updated, &items_deleted,
+				priv->cancellable, &error);
+		}
+
 		if (error)
 			break;
 
@@ -2340,11 +2387,11 @@ ebews_start_sync (gpointer data)
 		if (error)
 			break;
 
-		ews_book_backend_sqlitedb_set_sync_data (priv->ebsdb, priv->folder_id, sync_state, &error);
+		e_book_backend_sqlitedb_set_sync_data (priv->summary, priv->folder_id, sync_state, &error);
 	} while (!error && !includes_last_item);
 
 	if (!error)
-		ews_book_backend_sqlitedb_set_is_populated (priv->ebsdb, priv->folder_id, TRUE, &error);
+		e_book_backend_sqlitedb_set_is_populated (priv->summary, priv->folder_id, TRUE, &error);
 
 	g_free (sync_state);
 
@@ -2365,10 +2412,7 @@ delta_thread (gpointer data)
 {
 	EBookBackendEws *ebews = data;
 	EBookBackendEwsPrivate *priv = ebews->priv;
-	GTimeVal timeout;
-
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 0;
+	gint64 end_time;
 
 	while (TRUE)	{
 		gboolean succeeded;
@@ -2378,22 +2422,21 @@ delta_thread (gpointer data)
 		else
 			succeeded = ebews_start_gal_sync (ebews);
 
-		g_mutex_lock (priv->dlock->mutex);
+		g_mutex_lock (&priv->dlock->mutex);
 
 		if (!succeeded || priv->dlock->exit)
 			break;
 
-		g_get_current_time (&timeout);
-		g_time_val_add (&timeout, REFRESH_INTERVAL * 1000);
-		g_cond_timed_wait (priv->dlock->cond, priv->dlock->mutex, &timeout);
+		end_time = g_get_monotonic_time () + REFRESH_INTERVAL * G_TIME_SPAN_SECOND;
+		g_cond_wait_until (&priv->dlock->cond, &priv->dlock->mutex, end_time);
 
 		if (priv->dlock->exit)
 			break;
 
-		g_mutex_unlock (priv->dlock->mutex);
+		g_mutex_unlock (&priv->dlock->mutex);
 	}
 
-	g_mutex_unlock (priv->dlock->mutex);
+	g_mutex_unlock (&priv->dlock->mutex);
 	priv->dthread = NULL;
 	return NULL;
 }
@@ -2410,12 +2453,12 @@ fetch_deltas (EBookBackendEws *ebews)
 
 	if (!priv->dlock) {
 		priv->dlock = g_new0 (SyncDelta, 1);
-		priv->dlock->mutex = g_mutex_new ();
-		priv->dlock->cond = g_cond_new ();
+		g_mutex_init (&priv->dlock->mutex);
+		g_cond_init (&priv->dlock->cond);
 	}
 
 	priv->dlock->exit = FALSE;
-	priv->dthread = g_thread_create ((GThreadFunc) delta_thread, ebews, TRUE, &error);
+	priv->dthread = g_thread_try_new (NULL, (GThreadFunc) delta_thread, ebews, &error);
 	if (!priv->dthread) {
 		g_warning (G_STRLOC ": %s", error->message);
 		g_error_free (error);
@@ -2457,17 +2500,17 @@ fetch_from_offline (EBookBackendEws *ews,
 		return;
 	}
 
-	g_return_if_fail (priv->ebsdb != NULL);
+	g_return_if_fail (priv->summary != NULL);
 
-	contacts = ews_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
+	contacts = e_book_backend_sqlitedb_search (priv->summary, priv->folder_id, query, NULL, NULL, NULL, &error);
 	for (l = contacts; l != NULL; l = g_slist_next (l)) {
-		EwsSdbSearchData *s_data = (EwsSdbSearchData *) l->data;
+		EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
 
 		/* reset vcard to NULL as it would be free'ed in prefiltered_vcard function */
 		e_data_book_view_notify_update_prefiltered_vcard (book_view, s_data->uid, s_data->vcard);
 		s_data->vcard = NULL;
 
-		ews_book_backend_sqlitedb_search_data_free (s_data);
+		e_book_backend_sqlitedb_search_data_free (s_data);
 	}
 
 	if (contacts)
@@ -2507,7 +2550,7 @@ e_book_backend_ews_start_view (EBookBackend *backend,
 	e_data_book_view_notify_progress (book_view, -1, _("Searching..."));
 
 	if (!e_backend_get_online (E_BACKEND (backend))) {
-		if (priv->ebsdb && ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+		if (priv->summary && e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL)) {
 			fetch_from_offline (ebews, book_view, query, error);
 			return;
 		}
@@ -2538,8 +2581,8 @@ e_book_backend_ews_start_view (EBookBackend *backend,
 
 	ebews_start_refreshing (ebews);
 
-	if (priv->ebsdb &&
-	    ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+	if (priv->summary &&
+	    e_book_backend_sqlitedb_get_is_populated (priv->summary, priv->folder_id, NULL)) {
 		fetch_from_offline (ebews, book_view, query, error);
 		return;
 	}
@@ -2680,11 +2723,11 @@ e_book_backend_ews_load_source (EBookBackend *backend,
 	priv->folder_id = e_source_ews_folder_dup_id (
 		E_SOURCE_EWS_FOLDER (extension));
 
-	priv->ebsdb = ews_book_backend_sqlitedb_new (
+	priv->summary = e_book_backend_sqlitedb_new (
 		cache_dir, email, priv->folder_id,
 		display_name, TRUE, perror);
 
-	if (priv->ebsdb == NULL)
+	if (priv->summary == NULL)
 		return;
 
 	priv->marked_for_offline = FALSE;
@@ -2799,8 +2842,6 @@ e_book_backend_ews_get_backend_property (EBookBackend *backend,
 
 		g_slist_free (fields);
 		g_free (fields_str);
-	} else if (g_str_equal (prop_name, BOOK_BACKEND_PROPERTY_SUPPORTED_AUTH_METHODS)) {
-		e_data_book_respond_get_backend_property (book, opid, NULL, "plain/password");
 	} else {
 		E_BOOK_BACKEND_CLASS (e_book_backend_ews_parent_class)->get_backend_property (backend, book, opid, cancellable, prop_name);
 	}
@@ -2896,27 +2937,27 @@ e_book_backend_ews_dispose (GObject *object)
 	}
 
 	if (priv->dlock) {
-		g_mutex_lock (priv->dlock->mutex);
+		g_mutex_lock (&priv->dlock->mutex);
 		priv->dlock->exit = TRUE;
-		g_mutex_unlock (priv->dlock->mutex);
+		g_mutex_unlock (&priv->dlock->mutex);
 
-		g_cond_signal (priv->dlock->cond);
+		g_cond_signal (&priv->dlock->cond);
 
 		if (priv->dthread)
 			g_thread_join (priv->dthread);
 
-		g_mutex_free (priv->dlock->mutex);
-		g_cond_free (priv->dlock->cond);
+		g_mutex_clear (&priv->dlock->mutex);
+		g_cond_clear (&priv->dlock->cond);
 		g_free (priv->dlock);
 		priv->dthread = NULL;
 	}
 
-	if (priv->ebsdb) {
-		g_object_unref (priv->ebsdb);
-		priv->ebsdb = NULL;
+	if (priv->summary) {
+		g_object_unref (priv->summary);
+		priv->summary = NULL;
 	}
 
-	g_static_rec_mutex_free (&priv->rec_mutex);
+	g_rec_mutex_clear (&priv->rec_mutex);
 
 	g_free (priv);
 	priv = NULL;
@@ -3008,7 +3049,7 @@ e_book_backend_ews_init (EBookBackendEws *backend)
 	priv->ops = g_hash_table_new (NULL, NULL);
 
 	bews->priv = priv;
-	g_static_rec_mutex_init (&priv->rec_mutex);
+	g_rec_mutex_init (&priv->rec_mutex);
 	priv->cancellable = g_cancellable_new ();
 
 	g_signal_connect (
