@@ -78,14 +78,14 @@ which needs to be better organized via functions */
 	((obj), CAMEL_TYPE_EWS_FOLDER, CamelEwsFolderPrivate))
 
 struct _CamelEwsFolderPrivate {
-	GMutex *search_lock;	/* for locking the search object */
-	GStaticRecMutex cache_lock;	/* for locking the cache object */
+	GMutex search_lock;	/* for locking the search object */
+	GRecMutex cache_lock;	/* for locking the cache object */
 
 	/* For syncronizing refresh_info/sync_changes */
 	gboolean refreshing;
 	gboolean fetch_pending;
-	GMutex *state_lock;
-	GCond *fetch_cond;
+	GMutex state_lock;
+	GCond fetch_cond;
 	GHashTable *uid_eflags;
 };
 
@@ -178,7 +178,7 @@ camel_ews_folder_get_message_from_cache (CamelEwsFolder *ews_folder,
 
 	priv = ews_folder->priv;
 
-	g_static_rec_mutex_lock (&priv->cache_lock);
+	g_rec_mutex_lock (&priv->cache_lock);
 	stream = ews_data_cache_get (ews_folder->cache, "cur", uid, error);
 	if (!stream) {
 		gchar *old_fname = camel_data_cache_get_filename (
@@ -193,7 +193,7 @@ camel_ews_folder_get_message_from_cache (CamelEwsFolder *ews_folder,
 		}
 		g_free (old_fname);
 		if (!stream) {
-			g_static_rec_mutex_unlock (&priv->cache_lock);
+			g_rec_mutex_unlock (&priv->cache_lock);
 			return NULL;
 		}
 	}
@@ -206,7 +206,7 @@ camel_ews_folder_get_message_from_cache (CamelEwsFolder *ews_folder,
 		msg = NULL;
 	}
 
-	g_static_rec_mutex_unlock (&priv->cache_lock);
+	g_rec_mutex_unlock (&priv->cache_lock);
 	g_object_unref (stream);
 
 	return msg;
@@ -409,7 +409,7 @@ camel_ews_folder_get_message (CamelFolder *folder,
 	if (!camel_ews_store_connected (ews_store, cancellable, error))
 		return NULL;
 
-	g_mutex_lock (priv->state_lock);
+	g_mutex_lock (&priv->state_lock);
 
 	/* If another thread is already fetching this message, wait for it */
 
@@ -420,10 +420,10 @@ camel_ews_folder_get_message (CamelFolder *folder,
 	 * falls back to this function. */
 	if (g_hash_table_lookup (priv->uid_eflags, uid)) {
 		do {
-			g_cond_wait (priv->fetch_cond, priv->state_lock);
+			g_cond_wait (&priv->fetch_cond, &priv->state_lock);
 		} while (g_hash_table_lookup (priv->uid_eflags, uid));
 
-		g_mutex_unlock (priv->state_lock);
+		g_mutex_unlock (&priv->state_lock);
 
 		message = camel_ews_folder_get_message_from_cache (ews_folder, uid, cancellable, error);
 		return message;
@@ -433,7 +433,7 @@ camel_ews_folder_get_message (CamelFolder *folder,
 	 * we won't be inserting where an entry already exists. So it's
 	 * OK to insert uid itself, not g_strdup (uid) */
 	g_hash_table_insert (priv->uid_eflags, (gchar *) uid, (gchar *) uid);
-	g_mutex_unlock (priv->state_lock);
+	g_mutex_unlock (&priv->state_lock);
 
 	cnc = camel_ews_store_ref_connection (ews_store);
 	ids = g_slist_append (ids, (gchar *) uid);
@@ -545,10 +545,10 @@ camel_ews_folder_get_message (CamelFolder *folder,
 	message = camel_ews_folder_get_message_from_cache (ews_folder, uid, cancellable, error);
 
 exit:
-	g_mutex_lock (priv->state_lock);
+	g_mutex_lock (&priv->state_lock);
 	g_hash_table_remove (priv->uid_eflags, uid);
-	g_mutex_unlock (priv->state_lock);
-	g_cond_broadcast (priv->fetch_cond);
+	g_mutex_unlock (&priv->state_lock);
+	g_cond_broadcast (&priv->fetch_cond);
 
 	if (!message && !error)
 		g_set_error (
@@ -571,38 +571,6 @@ exit:
 	return message;
 }
 
-static void
-ews_folder_maybe_update_mlist (CamelFolder *folder,
-			       const gchar *uid,
-			       CamelMimeMessage *message)
-{
-	CamelEwsMessageInfo *mi;
-
-	g_return_if_fail (CAMEL_IS_FOLDER (folder));
-	g_return_if_fail (uid != NULL);
-	g_return_if_fail (message != NULL);
-
-	mi = (CamelEwsMessageInfo *) camel_folder_summary_get (folder->summary, uid);
-	if (!mi)
-		return;
-
-	if (!mi->info.mlist || !*mi->info.mlist) {
-		/* update mailing list information, if necessary */
-		gchar *mlist = camel_header_raw_check_mailing_list (&(CAMEL_MIME_PART (message)->headers));
-
-		if (mlist) {
-			if (mi->info.mlist)
-				camel_pstring_free (mi->info.mlist);
-			mi->info.mlist = camel_pstring_add (mlist, TRUE);
-			mi->info.dirty = TRUE;
-
-			camel_folder_summary_touch (folder->summary);
-		}
-	}
-
-	camel_message_info_free ((CamelMessageInfo *) mi);
-}
-
 /* Get the message from cache if available otherwise get it from server */
 static CamelMimeMessage *
 ews_folder_get_message_sync (CamelFolder *folder,
@@ -613,10 +581,8 @@ ews_folder_get_message_sync (CamelFolder *folder,
 	CamelMimeMessage *message;
 
 	message = camel_ews_folder_get_message_from_cache ((CamelEwsFolder *) folder, uid, cancellable, NULL);
-	if (!message) {
+	if (!message)
 		message = camel_ews_folder_get_message (folder, uid, EWS_ITEM_HIGH, cancellable, error);
-		ews_folder_maybe_update_mlist (folder, uid, message);
-	}
 
 	return message;
 }
@@ -642,12 +608,12 @@ ews_folder_search_by_expression (CamelFolder *folder,
 	ews_folder = CAMEL_EWS_FOLDER (folder);
 	priv = ews_folder->priv;
 
-	g_mutex_lock (priv->search_lock);
+	g_mutex_lock (&priv->search_lock);
 
 	camel_folder_search_set_folder (ews_folder->search, folder);
 	matches = camel_folder_search_search (ews_folder->search, expression, NULL, cancellable, error);
 
-	g_mutex_unlock (priv->search_lock);
+	g_mutex_unlock (&priv->search_lock);
 
 	return matches;
 }
@@ -665,12 +631,12 @@ ews_folder_count_by_expression (CamelFolder *folder,
 	ews_folder = CAMEL_EWS_FOLDER (folder);
 	priv = ews_folder->priv;
 
-	g_mutex_lock (priv->search_lock);
+	g_mutex_lock (&priv->search_lock);
 
 	camel_folder_search_set_folder (ews_folder->search, folder);
 	matches = camel_folder_search_count (ews_folder->search, expression, cancellable, error);
 
-	g_mutex_unlock (priv->search_lock);
+	g_mutex_unlock (&priv->search_lock);
 
 	return matches;
 }
@@ -692,12 +658,12 @@ ews_folder_search_by_uids (CamelFolder *folder,
 	if (uids->len == 0)
 		return g_ptr_array_new ();
 
-	g_mutex_lock (priv->search_lock);
+	g_mutex_lock (&priv->search_lock);
 
 	camel_folder_search_set_folder (ews_folder->search, folder);
 	matches = camel_folder_search_search (ews_folder->search, expression, uids, cancellable, error);
 
-	g_mutex_unlock (priv->search_lock);
+	g_mutex_unlock (&priv->search_lock);
 
 	return matches;
 }
@@ -714,11 +680,11 @@ ews_folder_search_free (CamelFolder *folder,
 
 	g_return_if_fail (ews_folder->search);
 
-	g_mutex_lock (priv->search_lock);
+	g_mutex_lock (&priv->search_lock);
 
 	camel_folder_search_free_result (ews_folder->search, uids);
 
-	g_mutex_unlock (priv->search_lock);
+	g_mutex_unlock (&priv->search_lock);
 
 	return;
 }
@@ -1405,6 +1371,45 @@ exit:
 	}
 }
 
+static void
+ews_folder_forget_all_mails (CamelEwsFolder *ews_folder)
+{
+	CamelFolder *folder;
+	CamelFolderChangeInfo *changes;
+	GPtrArray *known_uids;
+	gint ii;
+
+	g_return_if_fail (CAMEL_IS_EWS_FOLDER (ews_folder));
+
+	folder = CAMEL_FOLDER (ews_folder);
+	g_return_if_fail (folder != NULL);
+
+	known_uids = camel_folder_summary_get_array (folder->summary);
+	if (!known_uids)
+		return;
+
+	changes = camel_folder_change_info_new ();
+
+	camel_folder_summary_lock (folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
+	for (ii = 0; ii < known_uids->len; ii++) {
+		const gchar *uid = g_ptr_array_index (known_uids, ii);
+
+		camel_folder_change_info_remove_uid (changes, uid);
+		camel_folder_summary_remove_uid (folder->summary, uid);
+		ews_data_cache_remove (ews_folder->cache, "cur", uid, NULL);
+	}
+	camel_folder_summary_unlock (folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
+
+	if (camel_folder_change_info_changed (changes)) {
+		camel_folder_summary_touch (folder->summary);
+		camel_folder_summary_save_to_db (folder->summary, NULL);
+		camel_folder_changed (folder, changes);
+	}
+
+	camel_folder_change_info_free (changes);
+	camel_folder_summary_free_array (known_uids);
+}
+
 static gboolean
 ews_refresh_info_sync (CamelFolder *folder,
                        GCancellable *cancellable,
@@ -1429,15 +1434,15 @@ ews_refresh_info_sync (CamelFolder *folder,
 	if (!camel_ews_store_connected (ews_store, cancellable, error))
 		return FALSE;
 
-	g_mutex_lock (priv->state_lock);
+	g_mutex_lock (&priv->state_lock);
 
 	if (priv->refreshing) {
-		g_mutex_unlock (priv->state_lock);
+		g_mutex_unlock (&priv->state_lock);
 		return TRUE;
 	}
 
 	priv->refreshing = TRUE;
-	g_mutex_unlock (priv->state_lock);
+	g_mutex_unlock (&priv->state_lock);
 
 	cnc = camel_ews_store_ref_connection (ews_store);
 	g_return_val_if_fail (cnc != NULL, FALSE);
@@ -1459,14 +1464,21 @@ ews_refresh_info_sync (CamelFolder *folder,
 		GSList *items_deleted = NULL;
 		guint32 total, unread;
 
-		e_ews_connection_sync_folder_items_sync (
-			cnc, EWS_PRIORITY_MEDIUM,
-			sync_state, id,
-			"IdOnly", NULL,
-			EWS_MAX_FETCH_COUNT,
-			&sync_state, &includes_last_item,
-			&items_created, &items_updated,
-			&items_deleted, cancellable, &local_error);
+		e_ews_connection_sync_folder_items_sync (cnc, EWS_PRIORITY_MEDIUM, sync_state, id, "IdOnly", NULL, EWS_MAX_FETCH_COUNT,
+			&sync_state, &includes_last_item, &items_created, &items_updated, &items_deleted,
+			cancellable, &local_error);
+
+		if (g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_INVALIDSYNCSTATEDATA)) {
+			g_clear_error (&local_error);
+			g_free (((CamelEwsSummary *) folder->summary)->sync_state);
+			((CamelEwsSummary *) folder->summary)->sync_state = NULL;
+			sync_state = NULL;
+			ews_folder_forget_all_mails (ews_folder);
+
+			e_ews_connection_sync_folder_items_sync (cnc, EWS_PRIORITY_MEDIUM, NULL, id, "IdOnly", NULL, EWS_MAX_FETCH_COUNT,
+				&sync_state, &includes_last_item, &items_created, &items_updated, &items_deleted,
+				cancellable, &local_error);
+		}
 
 		if (local_error) {
 			camel_ews_store_maybe_disconnect (ews_store, local_error);
@@ -1512,9 +1524,9 @@ ews_refresh_info_sync (CamelFolder *folder,
 	if (local_error)
 		g_propagate_error (error, local_error);
 
-	g_mutex_lock (priv->state_lock);
+	g_mutex_lock (&priv->state_lock);
 	priv->refreshing = FALSE;
-	g_mutex_unlock (priv->state_lock);
+	g_mutex_unlock (&priv->state_lock);
 	if (sync_state != ((CamelEwsSummary *) folder->summary)->sync_state)
 		g_free (sync_state);
 	g_object_unref (cnc);
@@ -1811,9 +1823,9 @@ ews_folder_dispose (GObject *object)
 		ews_folder->search = NULL;
 	}
 
-	g_mutex_free (ews_folder->priv->search_lock);
+	g_mutex_clear (&ews_folder->priv->search_lock);
 	g_hash_table_destroy (ews_folder->priv->uid_eflags);
-	g_cond_free (ews_folder->priv->fetch_cond);
+	g_cond_clear (&ews_folder->priv->fetch_cond);
 
 	if (CAMEL_FOLDER (ews_folder)->summary)
 		g_signal_handlers_disconnect_by_func (CAMEL_FOLDER (ews_folder)->summary, G_CALLBACK (ews_folder_count_notify_cb), ews_folder);
@@ -1899,13 +1911,13 @@ camel_ews_folder_init (CamelEwsFolder *ews_folder)
 
 	folder->folder_flags = CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY;
 
-	ews_folder->priv->search_lock = g_mutex_new ();
-	ews_folder->priv->state_lock = g_mutex_new ();
-	g_static_rec_mutex_init (&ews_folder->priv->cache_lock);
+	g_mutex_init (&ews_folder->priv->search_lock);
+	g_mutex_init (&ews_folder->priv->state_lock);
+	g_rec_mutex_init (&ews_folder->priv->cache_lock);
 
 	ews_folder->priv->refreshing = FALSE;
 
-	ews_folder->priv->fetch_cond = g_cond_new ();
+	g_cond_init (&ews_folder->priv->fetch_cond);
 	ews_folder->priv->uid_eflags = g_hash_table_new (g_str_hash, g_str_equal);
 	camel_folder_set_lock_async (folder, TRUE);
 }
