@@ -23,7 +23,9 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <errno.h>
 #include <stdio.h>
@@ -36,8 +38,6 @@
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
-#include <libemail-engine/e-mail-folder-utils.h>
-
 #include "server/camel-ews-settings.h"
 #include "server/e-ews-item-change.h"
 #include "server/e-ews-message.h"
@@ -63,9 +63,9 @@
 
 struct _CamelEwsStorePrivate {
 	time_t last_refresh_time;
-	GMutex *get_finfo_lock;
+	GMutex get_finfo_lock;
 	EEwsConnection *connection;
-	GMutex *connection_lock;
+	GMutex connection_lock;
 };
 
 static gboolean	ews_store_construct	(CamelService *service, CamelSession *session,
@@ -360,10 +360,9 @@ ews_update_folder_hierarchy (CamelEwsStore *ews_store,
                              gboolean includes_last_folder,
                              GSList *folders_created,
                              GSList *folders_deleted,
-                             GSList *folders_updated,
-			     GSList **created_folder_ids)
+                             GSList *folders_updated)
 {
-	ews_utils_sync_folders (ews_store, folders_created, folders_deleted, folders_updated, created_folder_ids);
+	ews_utils_sync_folders (ews_store, folders_created, folders_deleted, folders_updated);
 	camel_ews_store_ensure_virtual_folders (ews_store);
 
 	camel_ews_store_summary_store_string_val (ews_store->summary, "sync_state", sync_state);
@@ -422,7 +421,7 @@ ews_disconnect_sync (CamelService *service,
 	CamelEwsStore *ews_store = (CamelEwsStore *) service;
 	CamelServiceClass *service_class;
 
-	g_mutex_lock (ews_store->priv->connection_lock);
+	g_mutex_lock (&ews_store->priv->connection_lock);
 
 	/* TODO cancel all operations in the connection */
 	if (ews_store->priv->connection != NULL) {
@@ -444,7 +443,7 @@ ews_disconnect_sync (CamelService *service,
 		ews_store->priv->connection = NULL;
 	}
 
-	g_mutex_unlock (ews_store->priv->connection_lock);
+	g_mutex_unlock (&ews_store->priv->connection_lock);
 
 	service_class = CAMEL_SERVICE_CLASS (camel_ews_store_parent_class);
 	return service_class->disconnect_sync (service, clean, cancellable, error);
@@ -484,13 +483,10 @@ ews_store_set_flags (CamelEwsStore *ews_store,
 	temp = folders;
 	while (temp != NULL) {
 		folder = (EEwsFolder *) temp->data;
+		fid = e_ews_folder_get_id (folder);
 
-		if (folder && !e_ews_folder_is_error (folder)) {
-			fid = e_ews_folder_get_id (folder);
-
-			if (camel_ews_store_summary_has_folder (ews_store->summary, fid->id))
-				camel_ews_store_summary_set_folder_flags (ews_store->summary, fid->id, system_folder[n].info_flags);
-		}
+		if (camel_ews_store_summary_has_folder (ews_store->summary, fid->id))
+			camel_ews_store_summary_set_folder_flags (ews_store->summary, fid->id, system_folder[n].info_flags);
 
 		temp = temp->next;
 		n++;
@@ -498,115 +494,196 @@ ews_store_set_flags (CamelEwsStore *ews_store,
 }
 
 static void
-ews_store_update_source_extension_folder (CamelEwsStore *ews_store,
-					  const gchar *folder_id,
-					  gpointer extension,
-					  const gchar *extension_property)
+ews_store_forget_all_folders (CamelEwsStore *ews_store)
 {
-	gchar *fullname;
+	CamelStore *store;
+	CamelSubscribable *subscribable;
+	GSList *folders, *l;
 
 	g_return_if_fail (CAMEL_IS_EWS_STORE (ews_store));
-	g_return_if_fail (extension != NULL);
-	g_return_if_fail (extension_property != NULL);
 
-	if (!folder_id)
+	store = CAMEL_STORE (ews_store);
+	subscribable = CAMEL_SUBSCRIBABLE (ews_store);
+	folders = camel_ews_store_summary_get_folders (ews_store->summary, NULL);
+
+	if (!folders)
 		return;
 
-	fullname = camel_ews_store_summary_get_folder_full_name (ews_store->summary, folder_id, NULL);
-	if (fullname) {
-		gchar *folder_uri;
+	for (l = folders; l != NULL; l = g_slist_next (l)) {
+		CamelFolderInfo *fi;
+		EEwsFolderType ftype;
 
-		folder_uri = e_mail_folder_uri_build (CAMEL_STORE (ews_store), fullname);
-		g_object_set (extension, extension_property, folder_uri, NULL);
-		g_free (folder_uri);
-		g_free (fullname);
+		ftype = camel_ews_store_summary_get_folder_type (ews_store->summary, l->data, NULL);
+		if (ftype != E_EWS_FOLDER_TYPE_MAILBOX)
+			continue;
+
+		fi = camel_ews_utils_build_folder_info (ews_store, l->data);
+		camel_subscribable_folder_unsubscribed (subscribable, fi);
+		camel_store_folder_deleted (store, fi);
+		camel_folder_info_free (fi);
+	}
+
+	g_slist_free_full (folders, g_free);
+}
+
+struct EwsUpdateForeignSubfoldersData
+{
+	CamelEwsStore *ews_store;
+	gchar *folder_id;
+};
+
+static void
+ews_update_foreign_subfolders_data_free (gpointer data)
+{
+	struct EwsUpdateForeignSubfoldersData *euf = data;
+
+	if (euf) {
+		g_object_unref (euf->ews_store);
+		g_free (euf->folder_id);
+		g_free (euf);
 	}
 }
 
 static void
-ews_store_maybe_update_sent_and_drafts (CamelEwsStore *ews_store,
-					/* const */ GSList *ews_folders)
+ews_store_update_foreign_subfolders (CamelSession *session,
+				     GCancellable *cancellable,
+				     gpointer user_data,
+				     GError **error)
 {
-	ESourceRegistry *registry;
-	ESource *sibling, *source = NULL;
+	struct EwsUpdateForeignSubfoldersData *euf = user_data;
+	CamelEwsStore *ews_store;
+	EEwsConnection *conn;
+	GSList *tocheck = NULL, *remote_folders = NULL, *local_folders = NULL;
+	const gchar *fid;
+	GError *local_error = NULL;
 
-	g_return_if_fail (CAMEL_IS_EWS_STORE (ews_store));
+	g_return_if_fail (euf != NULL);
 
-	registry = e_source_registry_new_sync (NULL, NULL);
-	g_return_if_fail (registry != NULL);
+	ews_store = euf->ews_store;
+	fid = euf->folder_id;
 
-	sibling = e_source_registry_ref_source (registry, camel_service_get_uid (CAMEL_SERVICE (ews_store)));
-	if (sibling) {
-		GList *sources, *siter;
+	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (ews_store)))
+		return;
 
-		sources = e_source_registry_list_sources (registry, E_SOURCE_EXTENSION_MAIL_SUBMISSION);
-		for (siter = sources; siter; siter = siter->next) {
-			source = siter->data;
+	conn = camel_ews_store_ref_connection (ews_store);
+	g_return_if_fail (conn != NULL);
 
-			if (!source || g_strcmp0 (e_source_get_parent (source), e_source_get_parent (sibling)) != 0 ||
-			    !e_source_has_extension (source, E_SOURCE_EXTENSION_MAIL_SUBMISSION) ||
-			    !e_source_has_extension (source, E_SOURCE_EXTENSION_MAIL_COMPOSITION))
-				source = NULL;
-			else
+	camel_operation_push_message (cancellable, _("Updating foreign folder structure"));
+
+	/* read remote folder structure at the server */
+	while (fid && !g_cancellable_is_cancelled (cancellable) && !local_error) {
+		gboolean includes_last_item = FALSE;
+		EwsFolderId *folder_id = e_ews_folder_id_new (fid, NULL, FALSE);
+
+		while (!includes_last_item && !g_cancellable_is_cancelled (cancellable) && !local_error) {
+			GSList *folders = NULL, *ff;
+
+			if (!e_ews_connection_find_folder_sync (conn, EWS_PRIORITY_MEDIUM, folder_id,
+				&includes_last_item, &folders, cancellable, &local_error))
 				break;
-		}
 
-		if (source &&
-		    e_source_has_extension (source, E_SOURCE_EXTENSION_MAIL_SUBMISSION) &&
-		    e_source_has_extension (source, E_SOURCE_EXTENSION_MAIL_COMPOSITION)) {
-			ESourceMailSubmission *subm_extension;
-			ESourceMailComposition *coms_extension;
-			gboolean changed = FALSE;
+			for (ff = folders; ff != NULL; ff = ff->next) {
+				EEwsFolder *folder = ff->data;
 
-			subm_extension = e_source_get_extension (source, E_SOURCE_EXTENSION_MAIL_SUBMISSION);
-			coms_extension = e_source_get_extension (source, E_SOURCE_EXTENSION_MAIL_COMPOSITION);
+				e_ews_folder_set_parent_id (folder, e_ews_folder_id_new (fid, NULL, FALSE));
 
-			if (e_source_mail_submission_get_sent_folder (subm_extension) &&
-			    g_ascii_strcasecmp (e_source_mail_submission_get_sent_folder (subm_extension), "folder://local/Sent") == 0) {
-				gchar *folder_id;
+				remote_folders = g_slist_prepend (remote_folders, folder);
 
-				folder_id = camel_ews_store_summary_get_folder_id_from_folder_type (ews_store->summary, CAMEL_FOLDER_TYPE_SENT);
-				if (folder_id) {
-					changed = TRUE;
-					ews_store_update_source_extension_folder (ews_store, folder_id, subm_extension, "sent-folder");
-					g_free (folder_id);
-				}
-			}
-
-			if (e_source_mail_composition_get_drafts_folder (coms_extension) &&
-			    g_ascii_strcasecmp (e_source_mail_composition_get_drafts_folder (coms_extension), "folder://local/Drafts") == 0) {
-				if (g_slist_length (ews_folders) == G_N_ELEMENTS (system_folder)) {
-					gint ii;
-					for (ii = 0; ii < G_N_ELEMENTS (system_folder); ii++) {
-						if (g_str_equal ("drafts", system_folder[ii].dist_folder_id)) {
-							break;
-						}
-					}
-
-					if (ii < G_N_ELEMENTS (system_folder)) {
-						EEwsFolder *drafts = g_slist_nth (ews_folders, ii)->data;
-						if (drafts && !e_ews_folder_is_error (drafts)) {
-							const EwsFolderId *fid = e_ews_folder_get_id (drafts);
-
-							if (fid && fid->id) {
-								changed = TRUE;
-								ews_store_update_source_extension_folder (ews_store, fid->id, coms_extension, "drafts-folder");
-							}
-						}
-					}
-				}
-			}
-
-			if (changed) {
-				e_source_write_sync (source, NULL, NULL);
+				if (e_ews_folder_get_child_count (folder) > 0 && e_ews_folder_get_id (folder))
+					tocheck = g_slist_prepend (tocheck, e_ews_folder_get_id (folder)->id);
 			}
 		}
 
-		g_list_free_full (sources, g_object_unref);
-		g_object_unref (sibling);
+		e_ews_folder_id_free (folder_id);
+
+		if (tocheck) {
+			fid = g_slist_last (tocheck)->data;
+			tocheck = g_slist_remove (tocheck, fid);
+		} else {
+			fid = NULL;
+		}
 	}
 
-	g_object_unref (registry);
+	/* get local folder structure */
+	if (!local_error && !g_cancellable_is_cancelled (cancellable)) {
+		gchar *full_name = camel_ews_store_summary_get_folder_full_name (ews_store->summary, euf->folder_id, NULL);
+		if (full_name) {
+			local_folders = camel_ews_store_summary_get_folders (ews_store->summary, full_name);
+		}
+		g_free (full_name);
+	}
+
+	/* merge local and remote folder structures */
+	if (!local_error && !g_cancellable_is_cancelled (cancellable)) {
+		GHashTable *locals = g_hash_table_new (g_str_hash, g_str_equal);
+		GSList *ii;
+
+		remote_folders = g_slist_reverse (remote_folders);
+
+		for (ii = local_folders; ii; ii = ii->next) {
+			g_hash_table_insert (locals, ii->data, ii->data);
+		}
+
+		for (ii = remote_folders; ii; ii = ii->next) {
+			EEwsFolder *folder = ii->data;
+			const EwsFolderId *folder_id = e_ews_folder_get_id (folder);
+			const EwsFolderId *parent_fid = e_ews_folder_get_parent_id (folder);
+
+			if (e_ews_folder_get_folder_type (folder) == E_EWS_FOLDER_TYPE_MAILBOX &&
+			    folder_id && folder_id->id) {
+				if (!g_hash_table_remove (locals, folder_id->id)) {
+					CamelFolderInfo *fi;
+
+					/* it's a new folder, add it */
+					camel_ews_store_summary_new_folder (
+						ews_store->summary,
+						folder_id->id, parent_fid ? parent_fid->id : euf->folder_id, folder_id->change_key,
+						e_ews_folder_get_name (folder), E_EWS_FOLDER_TYPE_MAILBOX,
+						CAMEL_FOLDER_SUBSCRIBED, e_ews_folder_get_total_count (folder), TRUE);
+
+					fi = camel_ews_utils_build_folder_info (ews_store, folder_id->id);
+					camel_store_folder_created (CAMEL_STORE (ews_store), fi);
+					camel_subscribable_folder_subscribed (CAMEL_SUBSCRIBABLE (ews_store), fi);
+					camel_folder_info_free (fi);
+				}
+			}
+		}
+
+		/* to not remove the parent */
+		g_hash_table_remove (locals, euf->folder_id);
+
+		/* and now the locals contains only folders which were removed */
+		if (g_hash_table_size (locals) > 0) {
+			CamelSubscribable *subscribable = CAMEL_SUBSCRIBABLE (ews_store);
+			CamelStore *store = CAMEL_STORE (ews_store);
+			GHashTableIter iter;
+			gpointer key, value;
+
+			g_hash_table_iter_init (&iter, locals);
+			while (g_hash_table_iter_next (&iter, &key, &value)) {
+				CamelFolderInfo *fi;
+
+				fi = camel_ews_utils_build_folder_info (ews_store, key);
+				camel_subscribable_folder_unsubscribed (subscribable, fi);
+				camel_store_folder_deleted (store, fi);
+				camel_folder_info_free (fi);
+			}
+		}
+
+		g_hash_table_destroy (locals);
+
+		camel_ews_store_summary_save (ews_store->summary, &local_error);
+	}
+
+	if (local_error)
+		g_propagate_error (error, local_error);
+
+	camel_operation_pop_message (cancellable);
+
+	g_slist_free_full (remote_folders, g_object_unref);
+	g_slist_free_full (local_folders, g_free);
+	g_slist_free (tocheck);
+	g_object_unref (conn);
 }
 
 static CamelAuthenticationResult
@@ -624,13 +701,13 @@ ews_authenticate_sync (CamelService *service,
 	GSList *folders_updated = NULL;
 	GSList *folders_deleted = NULL;
 	GSList *folder_ids = NULL, *folders = NULL;
-	GSList *created_folder_ids = NULL;
 	gboolean includes_last_folder = FALSE;
 	gboolean initial_setup = FALSE;
 	const gchar *password;
 	gchar *hosturl;
-	gchar *sync_state = NULL;
+	gchar *old_sync_state = NULL, *new_sync_state = NULL;
 	GError *local_error = NULL, *folder_err = NULL;
+	gint n = 0;
 
 	ews_store = CAMEL_EWS_STORE (service);
 
@@ -663,36 +740,74 @@ ews_authenticate_sync (CamelService *service,
 	 *     since we have to do that eventually anyway. */
 
 	/*use old sync_state from summary*/
-	sync_state = camel_ews_store_summary_get_string_val (ews_store->summary, "sync_state", NULL);
-	if (!sync_state)
+	old_sync_state = camel_ews_store_summary_get_string_val (ews_store->summary, "sync_state", NULL);
+	if (!old_sync_state)
 		initial_setup = TRUE;
 
-	e_ews_connection_sync_folder_hierarchy_sync (
-		connection, EWS_PRIORITY_MEDIUM,
-		&sync_state, &includes_last_folder,
-		&folders_created, &folders_updated, &folders_deleted,
+	e_ews_connection_sync_folder_hierarchy_sync (connection, EWS_PRIORITY_MEDIUM, old_sync_state,
+		&new_sync_state, &includes_last_folder, &folders_created, &folders_updated, &folders_deleted,
 		cancellable, &local_error);
 
+	g_free (old_sync_state);
+	old_sync_state = NULL;
+
+	if (!initial_setup && g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_INVALIDSYNCSTATEDATA)) {
+		g_clear_error (&local_error);
+		ews_store_forget_all_folders (ews_store);
+		camel_ews_store_summary_store_string_val (ews_store->summary, "sync_state", "");
+		camel_ews_store_summary_clear (ews_store->summary);
+
+		initial_setup = TRUE;
+
+		e_ews_connection_sync_folder_hierarchy_sync (connection, EWS_PRIORITY_MEDIUM, NULL,
+			&new_sync_state, &includes_last_folder, &folders_created, &folders_updated, &folders_deleted,
+			cancellable, &local_error);
+	}
+
 	if (local_error == NULL) {
-		g_mutex_lock (ews_store->priv->connection_lock);
+		GSList *foreign_fids, *ff;
+		CamelSession *session;
+
+		g_mutex_lock (&ews_store->priv->connection_lock);
 		if (ews_store->priv->connection != NULL)
 			g_object_unref (ews_store->priv->connection);
 		ews_store->priv->connection = g_object_ref (connection);
-		g_mutex_unlock (ews_store->priv->connection_lock);
+		g_mutex_unlock (&ews_store->priv->connection_lock);
 
 		/* This consumes all allocated result data. */
 		ews_update_folder_hierarchy (
-			ews_store, sync_state, includes_last_folder,
-			folders_created, folders_deleted, folders_updated, &created_folder_ids);
+			ews_store, new_sync_state, includes_last_folder,
+			folders_created, folders_deleted, folders_updated);
+
+		/* Also update folder structures of foreign folders,
+		   those which are subscribed with subfolders */
+		session = camel_service_get_session (service);
+		foreign_fids = camel_ews_store_summary_get_foreign_folders (ews_store->summary, NULL);
+		for (ff = foreign_fids; ff != NULL; ff = ff->next) {
+			const gchar *fid = ff->data;
+
+			if (camel_ews_store_summary_get_foreign_subfolders (ews_store->summary, fid, NULL)) {
+				struct EwsUpdateForeignSubfoldersData *euf;
+
+				euf = g_new0 (struct EwsUpdateForeignSubfoldersData, 1);
+				euf->ews_store = g_object_ref (ews_store);
+				euf->folder_id = g_strdup (fid);
+
+				camel_session_submit_job (session, ews_store_update_foreign_subfolders,
+					euf, ews_update_foreign_subfolders_data_free);
+			}
+		}
+
+		g_slist_free_full (foreign_fids, g_free);
 	} else {
-		g_mutex_lock (ews_store->priv->connection_lock);
+		g_mutex_lock (&ews_store->priv->connection_lock);
 		if (ews_store->priv->connection != NULL) {
 			g_object_unref (ews_store->priv->connection);
 			ews_store->priv->connection = NULL;
 		}
-		g_mutex_unlock (ews_store->priv->connection_lock);
+		g_mutex_unlock (&ews_store->priv->connection_lock);
 
-		g_free (sync_state);
+		g_free (new_sync_state);
 
 		/* Make sure we're not leaking anything. */
 		g_warn_if_fail (folders_created == NULL);
@@ -701,9 +816,7 @@ ews_authenticate_sync (CamelService *service,
 	}
 
 	/*get folders using distinguished id by GetFolder operation and set system flags to folders, only for first time*/
-	if (!local_error && (initial_setup || !camel_ews_settings_get_folders_initialized (ews_settings))) {
-		gint n = 0;
-
+	if (initial_setup && local_error == NULL) {
 		while (n < G_N_ELEMENTS (system_folder)) {
 			EwsFolderId *fid = NULL;
 
@@ -720,7 +833,7 @@ ews_authenticate_sync (CamelService *service,
 			NULL, folder_ids, &folders,
 			cancellable, &folder_err);
 
-		if (folders && (g_slist_length (folders) != G_N_ELEMENTS (system_folder)))
+		if (g_slist_length (folders) && (g_slist_length (folders) != G_N_ELEMENTS (system_folder)))
 			d (printf ("Error : not all folders are returned by getfolder operation"));
 		else if (folder_err == NULL && folders != NULL)
 			ews_store_set_flags (ews_store, folders);
@@ -730,12 +843,6 @@ ews_authenticate_sync (CamelService *service,
 		} else
 			d (printf ("folders for respective distinguished ids don't exist"));
 
-		if (!camel_ews_settings_get_folders_initialized (ews_settings)) {
-			ews_store_maybe_update_sent_and_drafts (ews_store, folders);
-			if (!folder_err)
-				camel_ews_settings_set_folders_initialized (ews_settings, TRUE);
-		}
-
 		g_slist_foreach (folders, (GFunc) g_object_unref, NULL);
 		g_slist_foreach (folder_ids, (GFunc) e_ews_folder_id_free, NULL);
 		g_slist_free (folders);
@@ -743,19 +850,6 @@ ews_authenticate_sync (CamelService *service,
 		g_clear_error (&folder_err);
 	}
 
-	/* postpone notification of new folders to time when also folder flags are known,
-	   thus the view in evolution sows Inbox with an Inbox icon. */
-	for (folder_ids = created_folder_ids; folder_ids; folder_ids = folder_ids->next) {
-		CamelFolderInfo *fi;
-
-		fi = camel_ews_utils_build_folder_info (ews_store, folder_ids->data);
-		camel_store_folder_created (CAMEL_STORE (ews_store), fi);
-		camel_subscribable_folder_subscribed (CAMEL_SUBSCRIBABLE (ews_store), fi);
-		camel_folder_info_free (fi);
-	}
-
-	g_slist_free_full (created_folder_ids, g_free);
-
 	if (local_error == NULL) {
 		result = CAMEL_AUTHENTICATION_ACCEPTED;
 	} else if (g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED)) {
@@ -907,18 +1001,18 @@ ews_folder_hierarchy_ready_cb (GObject *obj,
 
 		camel_ews_store_maybe_disconnect (ews_store, error);
 
-		g_mutex_lock (priv->get_finfo_lock);
+		g_mutex_lock (&priv->get_finfo_lock);
 		ews_store->priv->last_refresh_time -= FINFO_REFRESH_INTERVAL;
-		g_mutex_unlock (priv->get_finfo_lock);
+		g_mutex_unlock (&priv->get_finfo_lock);
 		goto exit;
 	}
-	g_mutex_lock (priv->get_finfo_lock);
+	g_mutex_lock (&priv->get_finfo_lock);
 	ews_update_folder_hierarchy (
 		ews_store, sync_state, includes_last_folder,
-		folders_created, folders_deleted, folders_updated, NULL);
+		folders_created, folders_deleted, folders_updated);
 
 	ews_store->priv->last_refresh_time = time (NULL);
-	g_mutex_unlock (priv->get_finfo_lock);
+	g_mutex_unlock (&priv->get_finfo_lock);
 
 exit:
 	g_object_unref (ews_store);
@@ -964,7 +1058,7 @@ ews_get_folder_info_sync (CamelStore *store,
 	CamelEwsStorePrivate *priv;
 	CamelFolderInfo *fi = NULL;
 	EEwsConnection *connection;
-	gchar *sync_state;
+	gchar *old_sync_state, *new_sync_state = NULL;
 	gboolean initial_setup = FALSE;
 	GSList *folders_created = NULL, *folders_updated = NULL;
 	GSList *folders_deleted = NULL;
@@ -984,36 +1078,50 @@ ews_get_folder_info_sync (CamelStore *store,
 	ews_store = (CamelEwsStore *) store;
 	priv = ews_store->priv;
 
-	g_mutex_lock (priv->get_finfo_lock);
+	g_mutex_lock (&priv->get_finfo_lock);
 	if (!(camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
 	      && camel_service_connect_sync ((CamelService *) store, cancellable, error))) {
 		camel_ews_store_ensure_virtual_folders (ews_store);
-		g_mutex_unlock (priv->get_finfo_lock);
+		g_mutex_unlock (&priv->get_finfo_lock);
 		goto offline;
 	}
 
-	sync_state = camel_ews_store_summary_get_string_val (ews_store->summary, "sync_state", NULL);
-	if (!sync_state)
+	old_sync_state = camel_ews_store_summary_get_string_val (ews_store->summary, "sync_state", NULL);
+	if (!old_sync_state)
 		initial_setup = TRUE;
 
-	if (!initial_setup && flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED) {
+	if (!initial_setup && (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED) != 0) {
 		time_t now = time (NULL);
 
-		g_free (sync_state);
+		g_free (old_sync_state);
 		if (now - priv->last_refresh_time > FINFO_REFRESH_INTERVAL && ews_refresh_finfo (ews_store))
 			ews_store->priv->last_refresh_time = time (NULL);
 
-		g_mutex_unlock (priv->get_finfo_lock);
+		g_mutex_unlock (&priv->get_finfo_lock);
 		goto offline;
 	}
 
 	connection = camel_ews_store_ref_connection (ews_store);
 
-	success = e_ews_connection_sync_folder_hierarchy_sync (
-		connection, EWS_PRIORITY_MEDIUM,
-		&sync_state, &includes_last_folder,
-		&folders_created, &folders_updated,
-		&folders_deleted, cancellable, &local_error);
+	success = e_ews_connection_sync_folder_hierarchy_sync (connection, EWS_PRIORITY_MEDIUM, old_sync_state,
+		&new_sync_state, &includes_last_folder, &folders_created, &folders_updated, &folders_deleted,
+		cancellable, &local_error);
+
+	g_free (old_sync_state);
+	old_sync_state = NULL;
+
+	if (!initial_setup && g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_INVALIDSYNCSTATEDATA)) {
+		g_clear_error (&local_error);
+		ews_store_forget_all_folders (ews_store);
+		camel_ews_store_summary_store_string_val (ews_store->summary, "sync_state", "");
+		camel_ews_store_summary_clear (ews_store->summary);
+
+		initial_setup = TRUE;
+
+		success = e_ews_connection_sync_folder_hierarchy_sync (connection, EWS_PRIORITY_MEDIUM, NULL,
+			&new_sync_state, &includes_last_folder, &folders_created, &folders_updated, &folders_deleted,
+			cancellable, &local_error);
+	}
 
 	g_object_unref (connection);
 
@@ -1028,13 +1136,13 @@ ews_get_folder_info_sync (CamelStore *store,
 		camel_ews_store_maybe_disconnect (ews_store, local_error);
 		g_propagate_error (error, local_error);
 
-		g_mutex_unlock (priv->get_finfo_lock);
+		g_mutex_unlock (&priv->get_finfo_lock);
 		return NULL;
 	}
 	ews_update_folder_hierarchy (
-		ews_store, sync_state, includes_last_folder,
-		folders_created, folders_deleted, folders_updated, NULL);
-	g_mutex_unlock (priv->get_finfo_lock);
+		ews_store, new_sync_state, includes_last_folder,
+		folders_created, folders_deleted, folders_updated);
+	g_mutex_unlock (&priv->get_finfo_lock);
 
 offline:
 	fi = folder_info_from_store_summary ( (CamelEwsStore *) store, top, flags, error);
@@ -1434,12 +1542,12 @@ camel_ews_store_ref_connection (CamelEwsStore *ews_store)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_STORE (ews_store), NULL);
 
-	g_mutex_lock (ews_store->priv->connection_lock);
+	g_mutex_lock (&ews_store->priv->connection_lock);
 
 	if (ews_store->priv->connection != NULL)
 		connection = g_object_ref (ews_store->priv->connection);
 
-	g_mutex_unlock (ews_store->priv->connection_lock);
+	g_mutex_unlock (&ews_store->priv->connection_lock);
 
 	return connection;
 }
@@ -1701,8 +1809,8 @@ ews_store_finalize (GObject *object)
 	ews_store = CAMEL_EWS_STORE (object);
 
 	g_free (ews_store->storage_path);
-	g_mutex_free (ews_store->priv->get_finfo_lock);
-	g_mutex_free (ews_store->priv->connection_lock);
+	g_mutex_clear (&ews_store->priv->get_finfo_lock);
+	g_mutex_clear (&ews_store->priv->connection_lock);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_ews_store_parent_class)->finalize (object);
@@ -1757,6 +1865,6 @@ camel_ews_store_init (CamelEwsStore *ews_store)
 		CAMEL_EWS_STORE_GET_PRIVATE (ews_store);
 
 	ews_store->priv->last_refresh_time = time (NULL) - (FINFO_REFRESH_INTERVAL + 10);
-	ews_store->priv->get_finfo_lock = g_mutex_new ();
-	ews_store->priv->connection_lock = g_mutex_new ();
+	g_mutex_init (&ews_store->priv->get_finfo_lock);
+	g_mutex_init (&ews_store->priv->connection_lock);
 }
