@@ -16,9 +16,12 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "e-ews-backend.h"
 
-#include <config.h>
 #include <glib/gi18n-lib.h>
 
 #include "server/e-source-ews-folder.h"
@@ -32,16 +35,16 @@ typedef struct _SyncFoldersClosure SyncFoldersClosure;
 struct _EEwsBackendPrivate {
 	/* Folder ID -> ESource */
 	GHashTable *folders;
-	GMutex *folders_lock;
+	GMutex folders_lock;
 
 	ESource *gal_source;
 	gchar *oal_selected;
 
 	gchar *sync_state;
-	GMutex *sync_state_lock;
+	GMutex sync_state_lock;
 
 	EEwsConnection *connection;
-	GMutex *connection_lock;
+	GMutex connection_lock;
 
 	gboolean need_update_folders;
 };
@@ -89,11 +92,11 @@ ews_backend_folders_contains (EEwsBackend *backend,
 
 	g_return_val_if_fail (folder_id != NULL, FALSE);
 
-	g_mutex_lock (backend->priv->folders_lock);
+	g_mutex_lock (&backend->priv->folders_lock);
 
 	contains = g_hash_table_contains (backend->priv->folders, folder_id);
 
-	g_mutex_unlock (backend->priv->folders_lock);
+	g_mutex_unlock (&backend->priv->folders_lock);
 
 	return contains;
 }
@@ -106,14 +109,14 @@ ews_backend_folders_insert (EEwsBackend *backend,
 	g_return_if_fail (folder_id != NULL);
 	g_return_if_fail (E_IS_SOURCE (source));
 
-	g_mutex_lock (backend->priv->folders_lock);
+	g_mutex_lock (&backend->priv->folders_lock);
 
 	g_hash_table_insert (
 		backend->priv->folders,
 		g_strdup (folder_id),
 		g_object_ref (source));
 
-	g_mutex_unlock (backend->priv->folders_lock);
+	g_mutex_unlock (&backend->priv->folders_lock);
 }
 
 static ESource *
@@ -124,14 +127,14 @@ ews_backend_folders_lookup (EEwsBackend *backend,
 
 	g_return_val_if_fail (folder_id != NULL, NULL);
 
-	g_mutex_lock (backend->priv->folders_lock);
+	g_mutex_lock (&backend->priv->folders_lock);
 
 	source = g_hash_table_lookup (backend->priv->folders, folder_id);
 
 	if (source != NULL)
 		g_object_ref (source);
 
-	g_mutex_unlock (backend->priv->folders_lock);
+	g_mutex_unlock (&backend->priv->folders_lock);
 
 	return source;
 }
@@ -144,11 +147,11 @@ ews_backend_folders_remove (EEwsBackend *backend,
 
 	g_return_val_if_fail (folder_id != NULL, FALSE);
 
-	g_mutex_lock (backend->priv->folders_lock);
+	g_mutex_lock (&backend->priv->folders_lock);
 
 	removed = g_hash_table_remove (backend->priv->folders, folder_id);
 
-	g_mutex_unlock (backend->priv->folders_lock);
+	g_mutex_unlock (&backend->priv->folders_lock);
 
 	return removed;
 }
@@ -369,12 +372,18 @@ ews_backend_add_gal_source (EEwsBackend *backend)
 	if (oal_selected != NULL) {
 		gchar *cp = strrchr (oal_selected, ':');
 		if (cp != NULL) {
-			*cp++ = '\0';
+			/* skip ':' and any leading backslash in the display name */
+			cp++;
+			while (*cp && *cp == '\\')
+				cp++;
+
 			display_name = cp;
 			oal_id = oal_selected;
 		} else {
 			g_free (oal_selected);
 			oal_selected = NULL;
+
+			camel_ews_settings_set_oal_selected (settings, NULL);
 		}
 	}
 
@@ -396,7 +405,7 @@ ews_backend_add_gal_source (EEwsBackend *backend)
 	e_source_set_display_name (source, display_name);
 
 	/* do not re-setup previously saved ESource,
-	   that would rewrite user's choice */
+	 * that would rewrite user's choice */
 	if (!e_source_has_extension (source, E_SOURCE_EXTENSION_EWS_FOLDER)) {
 		extension_name = E_SOURCE_EXTENSION_ADDRESS_BOOK;
 		backend_extension = e_source_get_extension (source, extension_name);
@@ -548,14 +557,14 @@ ews_backend_finalize (GObject *object)
 	priv = E_EWS_BACKEND_GET_PRIVATE (object);
 
 	g_hash_table_destroy (priv->folders);
-	g_mutex_free (priv->folders_lock);
+	g_mutex_clear (&priv->folders_lock);
 
 	g_free (priv->oal_selected);
 
 	g_free (priv->sync_state);
-	g_mutex_free (priv->sync_state_lock);
+	g_mutex_clear (&priv->sync_state_lock);
 
-	g_mutex_free (priv->connection_lock);
+	g_mutex_clear (&priv->connection_lock);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_ews_backend_parent_class)->finalize (object);
@@ -933,9 +942,9 @@ e_ews_backend_init (EEwsBackend *backend)
 		(GDestroyNotify) g_free,
 		(GDestroyNotify) g_object_unref);
 
-	backend->priv->folders_lock = g_mutex_new ();
-	backend->priv->sync_state_lock = g_mutex_new ();
-	backend->priv->connection_lock = g_mutex_new ();
+	g_mutex_init (&backend->priv->folders_lock);
+	g_mutex_init (&backend->priv->sync_state_lock);
+	g_mutex_init (&backend->priv->connection_lock);
 }
 
 void
@@ -983,10 +992,10 @@ e_ews_backend_ref_connection_sync (EEwsBackend *backend,
 
 	g_return_val_if_fail (E_IS_EWS_BACKEND (backend), NULL);
 
-	g_mutex_lock (backend->priv->connection_lock);
+	g_mutex_lock (&backend->priv->connection_lock);
 	if (backend->priv->connection != NULL)
 		connection = g_object_ref (backend->priv->connection);
-	g_mutex_unlock (backend->priv->connection_lock);
+	g_mutex_unlock (&backend->priv->connection_lock);
 
 	/* If we already have an authenticated
 	 * connection object, just return that. */
@@ -1004,11 +1013,11 @@ e_ews_backend_ref_connection_sync (EEwsBackend *backend,
 		cancellable, error);
 
 	if (success) {
-		g_mutex_lock (backend->priv->connection_lock);
+		g_mutex_lock (&backend->priv->connection_lock);
 		if (backend->priv->connection != NULL)
 			g_object_unref (backend->priv->connection);
 		backend->priv->connection = g_object_ref (connection);
-		g_mutex_unlock (backend->priv->connection_lock);
+		g_mutex_unlock (&backend->priv->connection_lock);
 	} else {
 		g_object_unref (connection);
 		connection = NULL;
@@ -1076,6 +1085,36 @@ ews_backend_sync_folders_thread (GSimpleAsyncResult *simple,
 		g_simple_async_result_take_error (simple, error);
 }
 
+static void
+ews_backend_delete_each_source_cb (gpointer data,
+				   gpointer user_data)
+{
+	ESource *source = data;
+	ECollectionBackend *backend = user_data;
+
+	e_collection_backend_delete_resource_sync (backend, source, NULL, NULL);
+}
+
+static void
+ews_backend_forget_all_sources (EEwsBackend *backend)
+{
+	GList *sources;
+
+	g_return_if_fail (E_IS_EWS_BACKEND (backend));
+
+	sources = e_collection_backend_list_calendar_sources (E_COLLECTION_BACKEND (backend));
+	g_list_foreach (sources, ews_backend_delete_each_source_cb, backend);
+	g_list_free_full (sources, g_object_unref);
+
+	sources = e_collection_backend_list_contacts_sources (E_COLLECTION_BACKEND (backend));
+	g_list_foreach (sources, ews_backend_delete_each_source_cb, backend);
+	g_list_free_full (sources, g_object_unref);
+
+	sources = e_collection_backend_list_mail_sources (E_COLLECTION_BACKEND (backend));
+	g_list_foreach (sources, ews_backend_delete_each_source_cb, backend);
+	g_list_free_full (sources, g_object_unref);
+}
+
 gboolean
 e_ews_backend_sync_folders_sync (EEwsBackend *backend,
                                  GCancellable *cancellable,
@@ -1086,8 +1125,9 @@ e_ews_backend_sync_folders_sync (EEwsBackend *backend,
 	GSList *folders_updated = NULL;
 	GSList *folders_deleted = NULL;
 	gboolean includes_last_folder = FALSE;
-	gchar *sync_state;
+	gchar *old_sync_state, *new_sync_state = NULL;
 	gboolean success;
+	GError *local_error = NULL;
 
 	g_return_val_if_fail (E_IS_EWS_BACKEND (backend), FALSE);
 
@@ -1101,17 +1141,34 @@ e_ews_backend_sync_folders_sync (EEwsBackend *backend,
 
 	backend->priv->need_update_folders = FALSE;
 
-	g_mutex_lock (backend->priv->sync_state_lock);
-	sync_state = g_strdup (backend->priv->sync_state);
-	g_mutex_unlock (backend->priv->sync_state_lock);
+	g_mutex_lock (&backend->priv->sync_state_lock);
+	old_sync_state = g_strdup (backend->priv->sync_state);
+	g_mutex_unlock (&backend->priv->sync_state_lock);
 
-	/* XXX I think this leaks the old sync_state value when
-	 *     it replaces it with the new sync_state value. */
-	success = e_ews_connection_sync_folder_hierarchy_sync (
-		connection, EWS_PRIORITY_MEDIUM,
-		&sync_state, &includes_last_folder,
-		&folders_created, &folders_updated, &folders_deleted,
-		cancellable, error);
+	success = e_ews_connection_sync_folder_hierarchy_sync (connection, EWS_PRIORITY_MEDIUM, old_sync_state,
+		&new_sync_state, &includes_last_folder, &folders_created, &folders_updated, &folders_deleted,
+		cancellable, &local_error);
+
+	if (old_sync_state && g_error_matches (local_error, EWS_CONNECTION_ERROR, EWS_CONNECTION_ERROR_INVALIDSYNCSTATEDATA)) {
+		g_clear_error (&local_error);
+
+		g_mutex_lock (&backend->priv->sync_state_lock);
+		g_free (backend->priv->sync_state);
+		backend->priv->sync_state = NULL;
+		g_mutex_unlock (&backend->priv->sync_state_lock);
+
+		ews_backend_forget_all_sources (backend);
+
+		success = e_ews_connection_sync_folder_hierarchy_sync (connection, EWS_PRIORITY_MEDIUM, NULL,
+			&new_sync_state, &includes_last_folder, &folders_created, &folders_updated, &folders_deleted,
+			cancellable, &local_error);
+	} else if (local_error) {
+		g_propagate_error (error, local_error);
+		local_error = NULL;
+	}
+
+	g_free (old_sync_state);
+	old_sync_state = NULL;
 
 	if (success) {
 		SyncFoldersClosure *closure;
@@ -1129,10 +1186,10 @@ e_ews_backend_sync_folders_sync (EEwsBackend *backend,
 			ews_backend_sync_folders_idle_cb, closure,
 			(GDestroyNotify) sync_folders_closure_free);
 
-		g_mutex_lock (backend->priv->sync_state_lock);
+		g_mutex_lock (&backend->priv->sync_state_lock);
 		g_free (backend->priv->sync_state);
-		backend->priv->sync_state = g_strdup (sync_state);
-		g_mutex_unlock (backend->priv->sync_state_lock);
+		backend->priv->sync_state = g_strdup (new_sync_state);
+		g_mutex_unlock (&backend->priv->sync_state_lock);
 
 	} else {
 		/* Make sure we're not leaking anything. */
@@ -1143,7 +1200,7 @@ e_ews_backend_sync_folders_sync (EEwsBackend *backend,
 		backend->priv->need_update_folders = TRUE;
 	}
 
-	g_free (sync_state);
+	g_free (new_sync_state);
 
 	g_object_unref (connection);
 
