@@ -16,6 +16,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "camel-ews-settings.h"
 
 #include <libedataserver/libedataserver.h>
@@ -25,18 +29,19 @@
 	((obj), CAMEL_TYPE_EWS_SETTINGS, CamelEwsSettingsPrivate))
 
 struct _CamelEwsSettingsPrivate {
-	GMutex *property_lock;
+	GMutex property_lock;
 	gboolean check_all;
 	gboolean filter_junk;
 	gboolean filter_junk_inbox;
 	gboolean oab_offline;
-	gboolean folders_initialized;
+	gboolean use_impersonation;
 	gchar *email;
 	gchar *gal_uid;
 	gchar *hosturl;
 	gchar *oaburl;
 	gchar *oal_selected;
 	guint timeout;
+	gchar *impersonate_user;
 };
 
 enum {
@@ -46,7 +51,6 @@ enum {
 	PROP_EMAIL,
 	PROP_FILTER_JUNK,
 	PROP_FILTER_JUNK_INBOX,
-	PROP_FOLDERS_INITIALIZED,
 	PROP_GAL_UID,
 	PROP_HOST,
 	PROP_HOSTURL,
@@ -56,7 +60,9 @@ enum {
 	PROP_PORT,
 	PROP_SECURITY_METHOD,
 	PROP_TIMEOUT,
-	PROP_USER
+	PROP_USER,
+	PROP_USE_IMPERSONATION,
+	PROP_IMPERSONATE_USER
 };
 
 G_DEFINE_TYPE_WITH_CODE (
@@ -103,12 +109,6 @@ ews_settings_set_property (GObject *object,
 				g_value_get_boolean (value));
 			return;
 
-		case PROP_FOLDERS_INITIALIZED:
-			camel_ews_settings_set_folders_initialized (
-				CAMEL_EWS_SETTINGS (object),
-				g_value_get_boolean (value));
-			return;
-
 		case PROP_GAL_UID:
 			camel_ews_settings_set_gal_uid (
 				CAMEL_EWS_SETTINGS (object),
@@ -168,6 +168,18 @@ ews_settings_set_property (GObject *object,
 				CAMEL_NETWORK_SETTINGS (object),
 				g_value_get_string (value));
 			return;
+
+		case PROP_USE_IMPERSONATION:
+			camel_ews_settings_set_use_impersonation (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_boolean (value));
+			return;
+
+		case PROP_IMPERSONATE_USER:
+			camel_ews_settings_set_impersonate_user (
+				CAMEL_EWS_SETTINGS (object),
+				g_value_get_string (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -215,13 +227,6 @@ ews_settings_get_property (GObject *object,
 				CAMEL_EWS_SETTINGS (object)));
 			return;
 
-		case PROP_FOLDERS_INITIALIZED:
-			g_value_set_boolean (
-				value,
-				camel_ews_settings_get_folders_initialized (
-				CAMEL_EWS_SETTINGS (object)));
-			return;
-
 		case PROP_GAL_UID:
 			g_value_take_string (
 				value,
@@ -291,6 +296,20 @@ ews_settings_get_property (GObject *object,
 				camel_network_settings_dup_user (
 				CAMEL_NETWORK_SETTINGS (object)));
 			return;
+
+		case PROP_USE_IMPERSONATION:
+			g_value_set_boolean (
+				value,
+				camel_ews_settings_get_use_impersonation (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
+
+		case PROP_IMPERSONATE_USER:
+			g_value_take_string (
+				value,
+				camel_ews_settings_dup_impersonate_user (
+				CAMEL_EWS_SETTINGS (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -303,13 +322,14 @@ ews_settings_finalize (GObject *object)
 
 	priv = CAMEL_EWS_SETTINGS_GET_PRIVATE (object);
 
-	g_mutex_free (priv->property_lock);
+	g_mutex_clear (&priv->property_lock);
 
 	g_free (priv->email);
 	g_free (priv->gal_uid);
 	g_free (priv->hosturl);
 	g_free (priv->oaburl);
 	g_free (priv->oal_selected);
+	g_free (priv->impersonate_user);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_ews_settings_parent_class)->finalize (object);
@@ -381,18 +401,6 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
 
-	g_object_class_install_property (
-		object_class,
-		PROP_FOLDERS_INITIALIZED,
-		g_param_spec_boolean (
-			"folders-initialized",
-			"Folders Initialized",
-			"Whether Sent Items and Drafts folders are initialized",
-			FALSE,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT |
-			G_PARAM_STATIC_STRINGS));
-
 	g_object_class_install_property (
 		object_class,
 		PROP_GAL_UID,
@@ -488,13 +496,37 @@ camel_ews_settings_class_init (CamelEwsSettingsClass *class)
 		object_class,
 		PROP_USER,
 		"user");
+
+	g_object_class_install_property (
+		object_class,
+		PROP_USE_IMPERSONATION,
+		g_param_spec_boolean (
+			"use-impersonation",
+			"Use Impersonation",
+			"Use Impersonation",
+			FALSE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_IMPERSONATE_USER,
+		g_param_spec_string (
+			"impersonate-user",
+			"Impersonate User",
+			"Impersonate User",
+			NULL,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT |
+			G_PARAM_STATIC_STRINGS));
 }
 
 static void
 camel_ews_settings_init (CamelEwsSettings *settings)
 {
 	settings->priv = CAMEL_EWS_SETTINGS_GET_PRIVATE (settings);
-	settings->priv->property_lock = g_mutex_new ();
+	g_mutex_init (&settings->priv->property_lock);
 }
 
 /**
@@ -538,50 +570,6 @@ camel_ews_settings_set_check_all (CamelEwsSettings *settings,
 	g_object_notify (G_OBJECT (settings), "check-all");
 }
 
-/**
- * camel_ews_settings_get_folders_initialized:
- * @settings: a #CamelEwsSettings
- *
- * Returns whether folders had been initialized.
- * This is used to know whether it's required to setup Drafts
- * and Sent Items folders as folders for that purpose in Evolution.
- *
- * Returns: whether folders had been initialized
- *
- * Since: 3.6
- **/
-gboolean
-camel_ews_settings_get_folders_initialized (CamelEwsSettings *settings)
-{
-	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), FALSE);
-
-	return settings->priv->folders_initialized;
-}
-
-/**
- * camel_ews_settings_set_folders_initialized:
- * @settings: a #CamelEwsSettings
- * @folders_initialized: whether folders had been initialized
- *
- * Sets whether to folders had been initialized. This is for Drafts
- * and Sent Items folders only.
- *
- * Since: 3.6
- **/
-void
-camel_ews_settings_set_folders_initialized (CamelEwsSettings *settings,
-					    gboolean folders_initialized)
-{
-	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
-
-	if ((settings->priv->folders_initialized ? 1 : 0) == (folders_initialized ? 1 : 0))
-		return;
-
-	settings->priv->folders_initialized = folders_initialized;
-
-	g_object_notify (G_OBJECT (settings), "folders-initialized");
-}
-
 const gchar *
 camel_ews_settings_get_email (CamelEwsSettings *settings)
 {
@@ -598,12 +586,12 @@ camel_ews_settings_dup_email (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	protected = camel_ews_settings_get_email (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	return duplicate;
 }
@@ -614,17 +602,17 @@ camel_ews_settings_set_email (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	if (g_strcmp0 (settings->priv->email, email) == 0) {
-		g_mutex_unlock (settings->priv->property_lock);
+		g_mutex_unlock (&settings->priv->property_lock);
 		return;
 	}
 
 	g_free (settings->priv->email);
 	settings->priv->email = e_util_strdup_strip (email);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	g_object_notify (G_OBJECT (settings), "email");
 }
@@ -731,12 +719,12 @@ camel_ews_settings_dup_gal_uid (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	protected = camel_ews_settings_get_gal_uid (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	return duplicate;
 }
@@ -747,17 +735,17 @@ camel_ews_settings_set_gal_uid (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	if (g_strcmp0 (settings->priv->gal_uid, gal_uid) == 0) {
-		g_mutex_unlock (settings->priv->property_lock);
+		g_mutex_unlock (&settings->priv->property_lock);
 		return;
 	}
 
 	g_free (settings->priv->gal_uid);
 	settings->priv->gal_uid = e_util_strdup_strip (gal_uid);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	g_object_notify (G_OBJECT (settings), "gal-uid");
 }
@@ -778,12 +766,12 @@ camel_ews_settings_dup_hosturl (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	protected = camel_ews_settings_get_hosturl (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	return duplicate;
 }
@@ -794,17 +782,17 @@ camel_ews_settings_set_hosturl (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	if (g_strcmp0 (settings->priv->hosturl, hosturl) == 0) {
-		g_mutex_unlock (settings->priv->property_lock);
+		g_mutex_unlock (&settings->priv->property_lock);
 		return;
 	}
 
 	g_free (settings->priv->hosturl);
 	settings->priv->hosturl = e_util_strdup_strip (hosturl);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	g_object_notify (G_OBJECT (settings), "hosturl");
 }
@@ -825,12 +813,12 @@ camel_ews_settings_dup_oaburl (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	protected = camel_ews_settings_get_oaburl (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	return duplicate;
 }
@@ -841,17 +829,17 @@ camel_ews_settings_set_oaburl (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	if (g_strcmp0 (settings->priv->oaburl, oaburl) == 0) {
-		g_mutex_unlock (settings->priv->property_lock);
+		g_mutex_unlock (&settings->priv->property_lock);
 		return;
 	}
 
 	g_free (settings->priv->oaburl);
 	settings->priv->oaburl = e_util_strdup_strip (oaburl);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	g_object_notify (G_OBJECT (settings), "oaburl");
 }
@@ -894,12 +882,12 @@ camel_ews_settings_dup_oal_selected (CamelEwsSettings *settings)
 
 	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	protected = camel_ews_settings_get_oal_selected (settings);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	return duplicate;
 }
@@ -910,17 +898,17 @@ camel_ews_settings_set_oal_selected (CamelEwsSettings *settings,
 {
 	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
 
-	g_mutex_lock (settings->priv->property_lock);
+	g_mutex_lock (&settings->priv->property_lock);
 
 	if (g_strcmp0 (settings->priv->oal_selected, oal_selected) == 0) {
-		g_mutex_unlock (settings->priv->property_lock);
+		g_mutex_unlock (&settings->priv->property_lock);
 		return;
 	}
 
 	g_free (settings->priv->oal_selected);
 	settings->priv->oal_selected = e_util_strdup_strip (oal_selected);
 
-	g_mutex_unlock (settings->priv->property_lock);
+	g_mutex_unlock (&settings->priv->property_lock);
 
 	g_object_notify (G_OBJECT (settings), "oal-selected");
 }
@@ -946,3 +934,72 @@ camel_ews_settings_set_timeout (CamelEwsSettings *settings,
 
 	g_object_notify (G_OBJECT (settings), "timeout");
 }
+
+gboolean
+camel_ews_settings_get_use_impersonation (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), FALSE);
+
+	return settings->priv->use_impersonation;
+}
+
+void
+camel_ews_settings_set_use_impersonation (CamelEwsSettings *settings,
+					  gboolean use_impersonation)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	if ((settings->priv->use_impersonation ? 1 : 0) == (use_impersonation ? 1 : 0))
+		return;
+
+	settings->priv->use_impersonation = use_impersonation;
+
+	g_object_notify (G_OBJECT (settings), "use-impersonation");
+}
+
+const gchar *
+camel_ews_settings_get_impersonate_user (CamelEwsSettings *settings)
+{
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
+
+	return settings->priv->impersonate_user;
+}
+
+gchar *
+camel_ews_settings_dup_impersonate_user (CamelEwsSettings *settings)
+{
+	const gchar *protected;
+	gchar *duplicate;
+
+	g_return_val_if_fail (CAMEL_IS_EWS_SETTINGS (settings), NULL);
+
+	g_mutex_lock (&settings->priv->property_lock);
+
+	protected = camel_ews_settings_get_impersonate_user (settings);
+	duplicate = g_strdup (protected);
+
+	g_mutex_unlock (&settings->priv->property_lock);
+
+	return duplicate;
+}
+
+void
+camel_ews_settings_set_impersonate_user (CamelEwsSettings *settings,
+					 const gchar *impersonate_user)
+{
+	g_return_if_fail (CAMEL_IS_EWS_SETTINGS (settings));
+
+	g_mutex_lock (&settings->priv->property_lock);
+
+	if (g_strcmp0 (settings->priv->impersonate_user, impersonate_user) == 0) {
+		g_mutex_unlock (&settings->priv->property_lock);
+		return;
+	}
+
+	g_free (settings->priv->impersonate_user);
+	settings->priv->impersonate_user = e_util_strdup_strip (impersonate_user);
+
+	g_mutex_unlock (&settings->priv->property_lock);
+
+	g_object_notify (G_OBJECT (settings), "impersonate-user");
+}
