@@ -203,6 +203,8 @@ const gchar *	e_ews_connection_get_password	(EEwsConnection *cnc);
 gchar *		e_ews_connection_dup_password	(EEwsConnection *cnc);
 void		e_ews_connection_set_password	(EEwsConnection *cnc,
 						 const gchar *password);
+const gchar *	e_ews_connection_get_impersonate_user
+						(EEwsConnection *cnc);
 CamelEwsSettings *
 		e_ews_connection_ref_settings	(EEwsConnection *cnc);
 SoupSession *	e_ews_connection_ref_soup_session
@@ -241,7 +243,7 @@ void		ews_delegate_info_free		(EwsDelegateInfo *info);
 void		e_ews_connection_sync_folder_items
 						(EEwsConnection *cnc,
 						 gint pri,
-						 const gchar *sync_state,
+						 const gchar *old_sync_state,
 						 const gchar *fid,
 						 const gchar *default_props,
 						 const gchar *additional_props,
@@ -252,7 +254,7 @@ void		e_ews_connection_sync_folder_items
 gboolean	e_ews_connection_sync_folder_items_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
-						 gchar **sync_state,
+						 gchar **new_sync_state,
 						 gboolean *includes_last_item,
 						 GSList **items_created,
 						 GSList **items_updated,
@@ -261,11 +263,12 @@ gboolean	e_ews_connection_sync_folder_items_finish
 gboolean	e_ews_connection_sync_folder_items_sync
 						(EEwsConnection *cnc,
 						 gint pri,
-						 gchar **sync_state,
+						 const gchar *old_sync_state,
 						 const gchar *fid,
 						 const gchar *default_props,
 						 const gchar *additional_props,
 						 guint max_entries,
+						 gchar **new_sync_state,
 						 gboolean *includes_last_item,
 						 GSList **items_created,
 						 GSList **items_updated,
@@ -460,7 +463,8 @@ gboolean	e_ews_connection_sync_folder_hierarchy_finish
 gboolean	e_ews_connection_sync_folder_hierarchy_sync
 						(EEwsConnection *cnc,
 						 gint pri,
-						 gchar **sync_state,
+						 const gchar *old_sync_state,
+						 gchar **new_sync_state,
 						 gboolean *includes_last_folder,
 						 GSList **folders_created,
 						 GSList **folders_updated,
@@ -717,7 +721,7 @@ GSList *	e_ews_connection_get_attachments_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *comp_uid,
-						 GSList *ids,
+						 const GSList *ids,
 						 const gchar *cache,
 						 gboolean include_mime,
 						 GSList **items,
@@ -963,6 +967,26 @@ gboolean	e_ews_connection_get_folder_info_sync
 						 EEwsFolder **folder,
 						 GCancellable *cancellable,
 						 GError **error);
+void		e_ews_connection_find_folder	(EEwsConnection *cnc,
+						 gint pri,
+						 const EwsFolderId *fid,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_find_folder_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 gboolean *includes_last_item,
+						 GSList **folders,
+						 GError **error);
+gboolean	e_ews_connection_find_folder_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const EwsFolderId *fid,
+						 gboolean *includes_last_item,
+						 GSList **folders,
+						 GCancellable *cancellable,
+						 GError **error);
 
 G_END_DECLS
 
