@@ -22,6 +22,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+
 #include <string.h>
 #include <glib/gi18n-lib.h>
 
@@ -35,7 +36,6 @@
 G_DEFINE_TYPE (EEwsFolder, e_ews_folder, G_TYPE_OBJECT)
 
 struct _EEwsFolderPrivate {
-	GError *error;
 	gchar *name;
 	EwsFolderId *fid;
 	EwsFolderId *parent_fid;
@@ -70,8 +70,6 @@ e_ews_folder_finalize (GObject *object)
 
 	priv = folder->priv;
 
-	g_clear_error (&priv->error);
-
 	if (priv->name) {
 		g_free (priv->name);
 		priv->name = NULL;
@@ -118,8 +116,7 @@ e_ews_folder_init (EEwsFolder *folder)
 	priv = g_new0 (EEwsFolderPrivate, 1);
 	folder->priv = priv;
 
-	priv->error = NULL;
-	priv->folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
+	priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
 	priv->foreign = FALSE;
 }
 
@@ -150,7 +147,19 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 
 	g_return_val_if_fail (param != NULL, FALSE);
 
-	if ((node = e_soap_parameter_get_first_child_by_name (param, "Folder")))
+	if (g_strcmp0 (e_soap_parameter_get_name (param), "Folder") == 0) {
+		node = param;
+		priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
+	} else if (g_strcmp0 (e_soap_parameter_get_name (param), "CalendarFolder") == 0) {
+		node = param;
+		priv->folder_type = E_EWS_FOLDER_TYPE_CALENDAR;
+	} else if (g_strcmp0 (e_soap_parameter_get_name (param), "ContactsFolder") == 0) {
+		node = param;
+		priv->folder_type = E_EWS_FOLDER_TYPE_CONTACTS;
+	} else if (g_strcmp0 (e_soap_parameter_get_name (param), "TasksFolder") == 0) {
+		node = param;
+		priv->folder_type = E_EWS_FOLDER_TYPE_TASKS;
+	} else if ((node = e_soap_parameter_get_first_child_by_name (param, "Folder")))
 		priv->folder_type = E_EWS_FOLDER_TYPE_MAILBOX;
 	else if ((node = e_soap_parameter_get_first_child_by_name (param, "CalendarFolder")))
 		priv->folder_type = E_EWS_FOLDER_TYPE_CALENDAR;
@@ -181,12 +190,6 @@ e_ews_folder_set_from_soap_parameter (EEwsFolder *folder,
 	if (subparam) {
 		value = e_soap_parameter_get_string_value (subparam);
 		e_ews_folder_set_folder_class (folder, (const gchar *) value);
-
-		if (value && priv->folder_type == E_EWS_FOLDER_TYPE_MAILBOX &&
-		    g_strcmp0 (value, "IPF.Note") != 0 && !g_str_has_prefix (value, "IPF.Note.")) {
-			priv->folder_type = E_EWS_FOLDER_TYPE_UNKNOWN;
-		}
-
 		g_free (value);
 	}
 
@@ -273,35 +276,6 @@ e_ews_folder_new_from_soap_parameter (ESoapParameter *param)
 	return folder;
 }
 
-EEwsFolder *
-e_ews_folder_new_from_error (const GError *error)
-{
-	EEwsFolder *folder;
-
-	g_return_val_if_fail (error != NULL, NULL);
-
-	folder = g_object_new (E_TYPE_EWS_FOLDER, NULL);
-	folder->priv->error = g_error_copy (error);
-
-	return folder;
-}
-
-gboolean
-e_ews_folder_is_error (EEwsFolder *folder)
-{
-	g_return_val_if_fail (E_IS_EWS_FOLDER (folder), TRUE);
-
-	return folder->priv->error != NULL;
-}
-
-const GError *
-e_ews_folder_get_error (EEwsFolder *folder)
-{
-	g_return_val_if_fail (E_IS_EWS_FOLDER (folder), NULL);
-
-	return folder->priv->error;
-}
-
 EwsFolderId *
 e_ews_folder_id_new (const gchar *id,
                      const gchar *change_key,
@@ -505,6 +479,7 @@ e_ews_folder_utils_populate_esource (ESource *source,
                                      const gchar *master_hosturl,
                                      const gchar *master_username,
                                      EEwsFolder *folder,
+				     gboolean include_subfolders,
                                      gboolean offline_sync,
                                      gint color_seed,
                                      GCancellable *cancellable,
@@ -550,19 +525,19 @@ e_ews_folder_utils_populate_esource (ESource *source,
 			ESourceEwsFolder *folder_ext;
 			ESourceOffline *offline_ext;
 
-			e_source_backend_set_backend_name (backend_ext, "ews");
+			e_source_backend_set_backend_name (backend_ext , "ews");
 
 			folder_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_EWS_FOLDER);
 			e_source_ews_folder_set_id (folder_ext, folder_id->id);
 			e_source_ews_folder_set_change_key (folder_ext, NULL);
 			e_source_ews_folder_set_foreign (folder_ext, e_ews_folder_get_foreign (folder));
+			e_source_ews_folder_set_foreign_subfolders (folder_ext, include_subfolders);
 
 			offline_ext = e_source_get_extension (source, E_SOURCE_EXTENSION_OFFLINE);
 			e_source_offline_set_stay_synchronized (offline_ext, offline_sync);
 
 			/* set also color for calendar-like sources */
 			if (folder_type != E_EWS_FOLDER_TYPE_CONTACTS) {
-				ESourceAlarms *alarms;
 				gchar *color_str;
 
 				color_str = e_ews_folder_utils_pick_color_spec (
@@ -570,9 +545,6 @@ e_ews_folder_utils_populate_esource (ESource *source,
 					folder_type != E_EWS_FOLDER_TYPE_CALENDAR);
 				e_source_selectable_set_color (E_SOURCE_SELECTABLE (backend_ext), color_str);
 				g_free (color_str);
-
-				alarms = e_source_get_extension (source, E_SOURCE_EXTENSION_ALARMS);
-				e_source_alarms_set_include_me (alarms, FALSE);
 			}
 
 			res = TRUE;
@@ -595,6 +567,7 @@ e_ews_folder_utils_add_as_esource (ESourceRegistry *pregistry,
                                    const gchar *master_hosturl,
                                    const gchar *master_username,
                                    EEwsFolder *folder,
+				   gboolean include_subfolders,
                                    gboolean offline_sync,
                                    gint color_seed,
                                    GCancellable *cancellable,
@@ -631,6 +604,7 @@ e_ews_folder_utils_add_as_esource (ESourceRegistry *pregistry,
 		master_hosturl,
 		master_username,
 		folder,
+		include_subfolders,
 		offline_sync,
 		color_seed,
 		cancellable,
