@@ -58,9 +58,6 @@ GType		e_ews_folder_get_type (void);
 const gchar *	e_ews_folder_type_to_nick (EEwsFolderType folder_type);
 EEwsFolderType	e_ews_folder_type_from_nick (const gchar *folder_type_nick);
 EEwsFolder *	e_ews_folder_new_from_soap_parameter (ESoapParameter *param);
-EEwsFolder *	e_ews_folder_new_from_error (const GError *error);
-gboolean	e_ews_folder_is_error (EEwsFolder *folder);
-const GError *	e_ews_folder_get_error (EEwsFolder *folder);
 const gchar *	e_ews_folder_get_name (EEwsFolder *folder);
 void		e_ews_folder_set_name (EEwsFolder *folder, const gchar *new_name);
 void		e_ews_folder_set_parent_id (EEwsFolder *folder, EwsFolderId *fid);
@@ -90,6 +87,7 @@ gboolean	e_ews_folder_utils_populate_esource		(ESource *source,
 								 const gchar *master_hosturl,
 								 const gchar *master_username,
 								 EEwsFolder *folder,
+								 gboolean include_subfolders,
 								 gboolean offline_sync,
 								 gint color_seed,
 								 GCancellable *cancellable,
@@ -98,6 +96,7 @@ gboolean	e_ews_folder_utils_add_as_esource		(ESourceRegistry *pregistry,
 								 const gchar *master_hosturl,
 								 const gchar *master_username,
 								 EEwsFolder *folder,
+								 gboolean include_subfolders,
 								 gboolean offline_sync,
 								 gint color_seed,
 								 GCancellable *cancellable,
